@@ -1,6 +1,6 @@
 /* MAPIStoreCalendarMessage.h - this file is part of SOGo
  *
- * Copyright (C) 2011 Inverse inc
+ * Copyright (C) 2011-2012 Inverse inc
  *
  * Author: Wolfgang Sourdeau <wsourdeau@inverse.ca>
  *
@@ -25,11 +25,14 @@
 
 #import "MAPIStoreGCSMessage.h"
 
+@class iCalCalendar;
+@class iCalEvent;
 @class MAPIStoreAppointmentWrapper;
 
 @interface MAPIStoreCalendarMessage : MAPIStoreGCSMessage
 {
-  MAPIStoreAppointmentWrapper *appointmentWrapper;
+  iCalCalendar *calendar;
+  iCalEvent *masterEvent;
 }
 
 @end
