@@ -1,8 +1,6 @@
 /* MAPIStoreMailFolder.m - this file is part of SOGo
  *
- * Copyright (C) 2011-2012 Inverse inc
- *
- * Author: Wolfgang Sourdeau <wsourdeau@inverse.ca>
+ * Copyright (C) 2011-2013 Inverse inc
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -61,7 +59,7 @@
 #import "NSData+MAPIStore.h"
 #import "NSString+MAPIStore.h"
 #import "SOGoMAPIDBMessage.h"
-#import "SOGoMAPIDBFolder.h"
+#import <SOGo/SOGoCacheGCSFolder.h>
 
 #import "MAPIStoreMailVolatileMessage.h"
 
@@ -108,7 +106,7 @@ static Class SOGoMailFolderK, MAPIStoreMailFolderK, MAPIStoreOutboxFolderK;
   ASSIGN (versionsMessage,
           [SOGoMAPIDBMessage objectWithName: @"versions.plist"
                                 inContainer: dbFolder]);
-  [versionsMessage setObjectType: MAPIDBObjectTypeInternal];
+  [versionsMessage setObjectType: MAPIInternalCacheObject];
 }
 
 - (BOOL) ensureFolderExists
@@ -1174,9 +1172,9 @@ _parseCOPYUID (NSString *line, NSArray **destUIDsP)
 
 - (MAPIStoreMessage *) createMessage
 {
-  SOGoMAPIObject *childObject;
+  SOGoCacheObject *childObject;
 
-  childObject = [SOGoMAPIObject objectWithName: [SOGoMAPIObject
+  childObject = [SOGoCacheObject objectWithName: [SOGoCacheObject
                                                   globallyUniqueObjectId]
                                    inContainer: sogoObject];
   return [MAPIStoreMailVolatileMessage
