@@ -38,7 +38,7 @@
 }
 
 + (struct mapistore_contexts_list *) listContextsForUser: (NSString *)  userName
-                                         withTDBIndexing: (struct tdb_wrap *) indexingTdb
+                                         withIndexing: (struct indexing_context *) indexing
                                                 inMemCtx: (TALLOC_CTX *) memCtx
 {
   struct mapistore_contexts_list *context;
