@@ -1,6 +1,6 @@
 /* NSDate+MAPIStore.h - this file is part of SOGo
  *
- * Copyright (C) 2010 Inverse inc.
+ * Copyright (C) 2010-2012 Inverse inc.
  *
  * Author: Wolfgang Sourdeau <wsourdeau@inverse.ca>
  *
@@ -25,9 +25,11 @@
 
 #import <Foundation/NSDate.h>
 
+@class NSCalendarDate;
+
 @interface NSDate (MAPIStoreDataTypes)
 
-+ (id) dateFromMinutesSince1601: (uint32_t) minutes;
++ (NSCalendarDate *) dateFromMinutesSince1601: (uint32_t) minutes;
 - (uint32_t) asMinutesSince1601;
 
 + (id) dateFromFileTime: (const struct FILETIME *) timeValue;
@@ -37,4 +39,6 @@
 
 @end
 
+NSComparisonResult NSDateCompare (id date1, id date2, void *);
+
 #endif /* NSCALENDARDATE+MAPISTORE_H */
