@@ -1,9 +1,6 @@
 /* SOGoComponentOccurence.m - this file is part of SOGo
  * 
- * Copyright (C) 2008-2011 Inverse inc.
- *
- * Author: Wolfgang Sourdeau <wsourdeau@inverse.ca>
- *         Francis Lachapelle <flachapelle@inverse.ca>
+ * Copyright (C) 2008-2014 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -130,6 +127,12 @@
   ASSIGN (parentCalendar, [component parent]);
 }
 
+
+- (iCalRepeatableEntityObject *) masterComponent
+{
+  return master;
+}
+
 - (void) setMasterComponent: (iCalRepeatableEntityObject *) newMaster
 {
   master = newMaster;
@@ -152,7 +155,6 @@
   NSArray *occurences;
   NSCalendarDate *recurrenceId, *currentId;
   NSException *error;
-  NSString *newContent;
   NSTimeZone *timeZone;
   iCalCalendar *calendar;
   iCalEntityObject *currentOccurence;
@@ -197,10 +199,8 @@
       [master addToExceptionDates: recurrenceId];
       [master increaseSequence];
 
-      // We generate the updated iCalendar file and we save it
-      // in the database.
-      newContent = [calendar versitString];
-      error = [container saveContentString: newContent];
+      // We save the updated iCalendar in the database.
+      error = [container saveCalendar: calendar];
     }
 
   return error;
@@ -215,6 +215,7 @@
   they should be siblings or SOGoComponentOccurence the parent class of SOGoCalendarComponent...
 - (NSException *) changeParticipationStatus: (NSString *) newStatus
                                withDelegate: (iCalPerson *) delegate
+                                      alarm: (iCalAlarm *) alarm
 {
   NSCalendarDate *date;
 
@@ -222,6 +223,7 @@
 
   return [container changeParticipationStatus: newStatus
                                  withDelegate: delegate
+                                        alarm: alarm
                               forRecurrenceId: date];
 }
 
