@@ -1,8 +1,6 @@
 /* iCalToDo+SOGo.h - this file is part of SOGo
  *
- * Copyright (C) 2007 Inverse inc.
- *
- * Author: Wolfgang Sourdeau <wsourdeau@inverse.ca>
+ * Copyright (C) 2007-2014 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,8 +27,6 @@
 
 @interface iCalToDo (SOGoExtensions)
 
-- (NSMutableDictionary *) quickRecord;
-
 @end
 
 #endif /* ICALTODO_SOGO_H */
