@@ -1,8 +1,6 @@
 /* SOGoContactFolders.h - this file is part of SOGo
  *
- * Copyright (C) 2006, 2007 Inverse inc.
- *
- * Author: Wolfgang Sourdeau <wsourdeau@inverse.ca>
+ * Copyright (C) 2006-2013 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,10 +25,19 @@
 
 @interface SOGoContactFolders : SOGoParentFolder
 
+- (NSString *) defaultFolderName;
+- (NSString *) collectedFolderName;
+
 - (NSException *) renameLDAPAddressBook: (NSString *) sourceID
                         withDisplayName: (NSString *) newDisplayName;
 - (NSException *) removeLDAPAddressBook: (NSString *) sourceID;
 
+- (NSDictionary *) systemSources;
+
+- (NSArray *) allContactsFromFilter: (NSString *) theFilter
+                      excludeGroups: (BOOL) excludeGroups
+                       excludeLists: (BOOL) excludeLists;
+
 @end
 
 #endif /* SOGOCONTACTFOLDERS_H */
