@@ -1,9 +1,6 @@
 /* SOGoContactLDIFEntry.m - this file is part of SOGo
  *
- * Copyright (C) 2006-2012 Inverse inc.
- *
- * Author: Wolfgang Sourdeau <wsourdeau@inverse.ca>
- *         Ludovic Marcotte <lmarcotte@inverse.ca>
+ * Copyright (C) 2006-2014 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -231,7 +228,7 @@
 				      [NSString stringWithFormat: @"%@.vcf", newUID]
 				    inContainer: newFolder];
 
-  return [newContact saveContentString: [newCard versitString]];
+  return [newContact saveComponent: newCard];
 }
 
 @end
