@@ -1,6 +1,6 @@
 /* NSString+Mail.h - this file is part of SOGo
  *
- * Copyright (C) 2007-2013 Inverse inc.
+ * Copyright (C) 2007-2014 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,12 +21,17 @@
 #ifndef NSSTRING_MAIL_H
 #define NSSTRING_MAIL_H
 
+#import <Foundation/NSArray.h>
 #import <Foundation/NSString.h>
 
 @interface NSString (SOGoExtension)
 
++ (NSString *) generateMessageID;
 - (NSString *) htmlToText;
+- (NSString *) htmlByExtractingImages: (NSMutableArray *) theImages;
 - (NSString *) stringByConvertingCRLNToHTML;
+- (int) indexOf: (unichar) _c
+      fromIndex: (int) start;
 - (int) indexOf: (unichar) _c;
 - (NSString *) decodedHeader;
 
