@@ -53,6 +53,7 @@
 {
   NSString *path;
   int IMAP4ID;
+  int sourceIMAP4ID;
   NSMutableDictionary *headers;
   NSString *inReplyTo;
   NSString *text;
@@ -87,6 +88,9 @@
 - (void) setSourceFolder: (NSString *) newSourceFolder;
 - (NSString *) sourceFolder;
 
+- (void) setSourceIMAP4ID: (int) newSourceIMAPID;
+- (int) sourceIMAP4ID;
+
 - (void) setIMAP4ID: (int) newIMAPID;
 - (int) IMAP4ID;
 
