@@ -45,6 +45,7 @@
 #import <NGImap4/NGImap4Client.h>
 #import <NGImap4/NGImap4Envelope.h>
 #import <NGImap4/NGImap4EnvelopeAddress.h>
+#import <NGMail/NGMailAddress.h>
 #import <NGMail/NGMailAddressParser.h>
 #import <NGMail/NGMimeMessage.h>
 #import <NGMail/NGMimeMessageGenerator.h>
@@ -197,6 +198,7 @@ static NSString    *userAgent      = nil;
 {
   if ((self = [super init]))
     {
+      sourceIMAP4ID = -1;
       IMAP4ID = -1;
       headers = [NSMutableDictionary new];
       text = @"";
@@ -234,8 +236,7 @@ static NSString    *userAgent      = nil;
 {
   if (!path)
     {
-      path = [[self userSpoolFolderPath] stringByAppendingPathComponent:
-					   nameInContainer];
+      path = [[self userSpoolFolderPath] stringByAppendingPathComponent: nameInContainer];
       [path retain];
     }
 
@@ -466,6 +467,9 @@ static NSString    *userAgent      = nil;
                 forKey: @"isHTML"];
       if (inReplyTo)
 	[infos setObject: inReplyTo forKey: @"inReplyTo"];
+      if (sourceIMAP4ID > -1)
+	[infos setObject: [NSString stringWithFormat: @"%i", sourceIMAP4ID]
+		  forKey: @"sourceIMAP4ID"];
       if (IMAP4ID > -1)
 	[infos setObject: [NSString stringWithFormat: @"%i", IMAP4ID]
 		  forKey: @"IMAP4ID"];
@@ -513,6 +517,10 @@ static NSString    *userAgent      = nil;
     [self setText: value];
   isHTML = [[infoDict objectForKey: @"isHTML"] boolValue];
 
+  value = [infoDict objectForKey: @"sourceIMAP4ID"];
+  if (value)
+    [self setSourceIMAP4ID: [value intValue]];
+
   value = [infoDict objectForKey: @"IMAP4ID"];
   if (value)
     [self setIMAP4ID: [value intValue]];
@@ -564,6 +572,22 @@ static NSString    *userAgent      = nil;
     [self debugWithFormat: @"Note: info object does not yet exist: %@", p];
 }
 
+//
+//
+//
+- (void) setSourceIMAP4ID: (int) newSourceIMAP4ID
+{
+  sourceIMAP4ID = newSourceIMAP4ID;
+}
+
+//
+//
+//
+- (int) sourceIMAP4ID
+{
+  return sourceIMAP4ID;
+}
+
 //
 //
 //
@@ -605,12 +629,12 @@ static NSString    *userAgent      = nil;
   
   folder = [imap4 imap4FolderNameForURL: [container imap4URL]];
   result = [client append: message toFolder: folder
-                withFlags: [NSArray arrayWithObjects: @"seen", @"draft", nil]];
+                withFlags: [NSArray arrayWithObjects: @"draft", nil]];
   if ([[result objectForKey: @"result"] boolValue])
     {
       if (IMAP4ID > -1)
 	error = [imap4 markURLDeleted: [self imap4URL]];
-      IMAP4ID = [self IMAP4IDFromAppendResult: result];
+      [self setIMAP4ID: [self IMAP4IDFromAppendResult: result]];
       if (imap4URL)
         {
           // Invalidate the IMAP message URL since the message ID has changed
@@ -902,9 +926,7 @@ static NSString    *userAgent      = nil;
   [self setHeaders: info];
 
   [self setText: [sourceMail contentForEditing]];
-  [self setSourceURL: [sourceMail imap4URLString]];
   [self setIMAP4ID: [[sourceMail nameInContainer] intValue]];
-  [self setSourceFolderWithMailObject: sourceMail];
 
   [self storeInfo];
 }
@@ -934,7 +956,7 @@ static NSString    *userAgent      = nil;
   [self setHeaders: info];
   [self setSourceURL: [sourceMail imap4URLString]];
   [self setSourceFlag: @"Answered"];
-  [self setIMAP4ID: [[sourceMail nameInContainer] intValue]];
+  [self setSourceIMAP4ID: [[sourceMail nameInContainer] intValue]];
   [self setSourceFolderWithMailObject: sourceMail];
 
   [self storeInfo];
@@ -957,7 +979,7 @@ static NSString    *userAgent      = nil;
   
   [self setSourceURL: [sourceMail imap4URLString]];
   [self setSourceFlag: @"$Forwarded"];
-  [self setIMAP4ID: [[sourceMail nameInContainer] intValue]];
+  [self setSourceIMAP4ID: [[sourceMail nameInContainer] intValue]];
   [self setSourceFolderWithMailObject: sourceMail];
 
   /* attach message */
@@ -1195,13 +1217,13 @@ static NSString    *userAgent      = nil;
   
   if (!isHTML)
     {
-      [map setObject: contentTypeValue  forKey: @"content-type"];
+      [message setHeader: contentTypeValue  forKey: @"content-type"];
       body = text;
     }
   else
     {
       body = [[[NGMimeMultipartBody alloc] initWithPart: message] autorelease];
-      [map addObject: MultiAlternativeType forKey: @"content-type"];
+      [message setHeader: MultiAlternativeType forKey: @"content-type"];
 
       // Get the text part from it and add it
       [body addBodyPart: [self plainTextBodyPartForText]];
@@ -1760,62 +1782,62 @@ static NSString    *userAgent      = nil;
 - (NSException *) sendMail
 {
   SOGoUserDefaults *ud;
+  
   ud = [[context activeUser] userDefaults];
-
+  
   if ([ud mailAddOutgoingAddresses])
-  {
-    SOGoContactFolders *contactFolders;
-    NGMailAddressParser *parser;
-    id parsedRecipient;
-    SOGoContactFolder *folder;
-    SOGoContactGCSEntry *newContact;
-    NGVCard *card;
-    Class contactGCSEntry;
-    NSMutableArray *recipients;
-    NSString *recipient, *emailAddress, *addressBook, *uid;
-    NSArray *matchingContacts;
-    int i;
-
-    // Get all the addressbooks
-    contactFolders = [[[context activeUser] homeFolderInContext: context]
-                                                     lookupName: @"Contacts"
-                                                      inContext: context
-                                                      acquire: NO];
-    // Get all the recipients from the current email
-    recipients = [self allRecipients];
-    for (i = 0; i < [recipients count]; i++)
     {
-      // The address contains a string. ex: "John Doe <sogo1@exemple.com>"
-      recipient = [recipients objectAtIndex: i];
-      parser = [NGMailAddressParser mailAddressParserWithString: recipient];
-      parsedRecipient = [parser parse];
-      emailAddress = [parsedRecipient address];
-
-      matchingContacts = [contactFolders allContactsFromFilter: emailAddress
-                                                 excludeGroups: YES
-                                                  excludeLists: YES];
-    }
-    // If we don't get any results from the autocompletion code, we add it..
-    if ([matchingContacts count] == 0)
-    {
-      // Get the selected addressbook from the user preferences where the new address will be added
-      addressBook = [ud selectedAddressBook];
-      folder = [contactFolders lookupName: addressBook inContext: context  acquire: NO];
-      uid = [folder globallyUniqueObjectId];
+      NSString *recipient, *emailAddress, *addressBook, *uid;
+      NSArray *matchingContacts, *recipients;
+      SOGoContactFolders *contactFolders;
+      SOGoContactGCSEntry *newContact;
+      NGMailAddress *parsedRecipient;
+      NGMailAddressParser *parser;
+      SOGoContactFolder *folder;
+      NGVCard *card;
+ 
+      int i;
+    
+      // Get all the addressbooks
+      contactFolders = [[[context activeUser] homeFolderInContext: context]
+                         lookupName: @"Contacts"
+                          inContext: context
+                            acquire: NO];
+      // Get all the recipients from the current email
+      recipients = [self allRecipients];
+      for (i = 0; i < [recipients count]; i++)
+        {
+          // The address contains a string. ex: "John Doe <sogo1@exemple.com>"
+          recipient = [recipients objectAtIndex: i];
+          parser = [NGMailAddressParser mailAddressParserWithString: recipient];
+          parsedRecipient = [parser parse];
+          emailAddress = [parsedRecipient address];
       
-      if (folder && uid)
-      {
-        card = [NGVCard cardWithUid: uid];
-        [card addEmail: emailAddress types: nil];
-        
-        contactGCSEntry = NSClassFromString(@"SOGoContactGCSEntry");
-        newContact = [contactGCSEntry objectWithName: uid
-                                         inContainer: folder];
-        [newContact setIsNew: YES];
-        [newContact saveContentString: [card versitString]];
-      }
+          matchingContacts = [contactFolders allContactsFromFilter: emailAddress
+                                                     excludeGroups: YES
+                                                      excludeLists: YES];
+        }
+      // If we don't get any results from the autocompletion code, we add it..
+      if ([matchingContacts count] == 0)
+        {
+          // Get the selected addressbook from the user preferences where the new address will be added
+          addressBook = [ud selectedAddressBook];
+          folder = [contactFolders lookupName: addressBook inContext: context  acquire: NO];
+          uid = [folder globallyUniqueObjectId];
+      
+          if (folder && uid)
+            {
+              card = [NGVCard cardWithUid: uid];
+              [card addEmail: emailAddress types: nil];
+              [card setFn: [parsedRecipient displayName]];
+              
+              newContact = [SOGoContactGCSEntry objectWithName: uid
+                                                   inContainer: folder];
+              [newContact setIsNew: YES];
+              [newContact saveComponent: card];
+            }
+        }
     }
-  }
   return [self sendMailAndCopyToSent: YES];
 }
 
