@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2009-2011 Inverse inc.
+  Copyright (C) 2009-2014 Inverse inc.
   Copyright (C) 2004-2005 SKYRIX Software AG
 
   This file is part of SOGo
@@ -94,6 +94,12 @@
 
 - (NSCalendarDate *) mostRecentMessageDate;
 
+- (NSString *) davCollectionTag;
+
+- (NSArray *) syncTokenFieldsWithProperties: (NSDictionary *) properties
+                          matchingSyncToken: (NSString *) syncToken
+                                   fromDate: (NSCalendarDate *) theStartDate;
+
 /* flags */
 
 - (NSException *) addFlagsToAllMessages: (id) _f;
