@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2009-2013 Inverse inc.
+  Copyright (C) 2009-2014 Inverse inc.
   Copyright (C) 2004-2005 SKYRIX Software AG
 
   This file is part of SOGo.
@@ -24,6 +24,7 @@
 #import <Foundation/NSDictionary.h>
 #import <Foundation/NSEnumerator.h>
 #import <Foundation/NSURL.h>
+#import <Foundation/NSValue.h>
 #import <Foundation/NSTask.h>
 
 #import <NGObjWeb/NSException+HTTP.h>
@@ -43,6 +44,7 @@
 
 #import <NGImap4/NGImap4Connection.h>
 #import <NGImap4/NGImap4Client.h>
+#import <NGImap4/NSString+Imap4.h>
 
 #import <SOGo/DOMNode+SOGo.h>
 #import <SOGo/NSArray+Utilities.h>
@@ -70,6 +72,28 @@
 
 static NSString *defaultUserID =  @"anyone";
 
+static NSComparisonResult
+_compareFetchResultsByMODSEQ (id entry1, id entry2, void *data)
+{
+  static NSNumber *zeroNumber = nil;
+  NSNumber *modseq1, *modseq2;
+
+  if (!zeroNumber)
+    {
+      zeroNumber = [NSNumber numberWithUnsignedLongLong: 0];
+      [zeroNumber retain];
+    }
+
+  modseq1 = [entry1 objectForKey: @"modseq"];
+  if (!modseq1)
+    modseq1 = zeroNumber;
+  modseq2 = [entry2 objectForKey: @"modseq"];
+  if (!modseq2)
+    modseq2 = zeroNumber;
+
+  return [modseq1 compare: modseq2];
+}
+
 @interface NGImap4Connection (PrivateMethods)
 
 - (NSString *) imap4FolderNameForURL: (NSURL *) url;
@@ -289,22 +313,30 @@ static NSString *defaultUserID =  @"anyone";
           path = [[imap4URL path] stringByDeletingLastPathComponent];
           if (![path hasSuffix: @"/"])
             path = [path stringByAppendingString: @"/"];
-          destURL = [[NSURL alloc] initWithScheme: [imap4URL scheme]
-                                             host: [imap4URL host]
-                                             path: [NSString stringWithFormat: @"%@%@",
-                                                             path, newName]];
+
+	  // If new name contains the path - dont't need to add
+          if ([newName rangeOfString: @"/"].location == NSNotFound)
+            destURL = [[NSURL alloc] initWithScheme: [imap4URL scheme]
+                                               host: [imap4URL host]
+                                               path: [NSString stringWithFormat: @"%@%@",
+                                                               path, [newName stringByEncodingImap4FolderName]]];
+          else
+            destURL = [[NSURL alloc] initWithScheme: [imap4URL scheme]
+                                               host: [imap4URL host]
+                                               path: [NSString stringWithFormat: @"%@",
+                                                               [newName stringByEncodingImap4FolderName]]];
           [destURL autorelease];
           error = [imap4 moveMailboxAtURL: imap4URL
                                     toURL: destURL];
           if (!error)
             {
-              ASSIGN (imap4URL, nil);
-              ASSIGN (nameInContainer,
-                      ([NSString stringWithFormat: @"folder%@", [newName asCSSIdentifier]]));
-
               // We unsubscribe to the old one, and subscribe back to the new one
               [client subscribe: [destURL path]];
               [client unsubscribe: [imap4URL path]];
+
+              ASSIGN (imap4URL, nil);
+              ASSIGN (nameInContainer,
+                      ([NSString stringWithFormat: @"folder%@", [newName asCSSIdentifier]]));
             }
         }
       else
@@ -614,20 +646,19 @@ static NSString *defaultUserID =  @"anyone";
   if (max > 1)
     {
       currentAccountName = [[self mailAccountFolder] nameInContainer];
-      if ([[folders objectAtIndex: 1] isEqualToString: currentAccountName])
-        {
-          for (count = 2; count < max; count++)
-            {
-              currentFolderName
-                = [[folders objectAtIndex: count] substringFromIndex: 6];
-              [imapDestinationFolder appendFormat: @"/%@", currentFolderName];
-            }
+      client = [[self imap4Connection] client];
+      [imap4 selectFolder: [self imap4URL]];
 
-          client = [[self imap4Connection] client];
-          if (client)
+      for (count = 2; count < max; count++)
+        {
+          currentFolderName = [[folders objectAtIndex: count] substringFromIndex: 6];
+          [imapDestinationFolder appendFormat: @"/%@", currentFolderName];
+        }
+
+      if (client)
+        {
+          if ([[folders objectAtIndex: 1] isEqualToString: currentAccountName])
             {
-              [imap4 selectFolder: [self imap4URL]];
-  
               // We make sure the destination IMAP folder exist, if not, we create it.
               result = [[client status: imapDestinationFolder
                                  flags: [NSArray arrayWithObject: @"UIDVALIDITY"]]
@@ -647,13 +678,73 @@ static NSString *defaultUserID =  @"anyone";
                                                                  objectForKey: @"description"]];
             }
           else
-            result = [NSException exceptionWithName: @"SOGoMailException"
-                                             reason: @"IMAP connection is invalid"
-                                           userInfo: nil];
+            {
+              // Destination folder is in a different account
+              SOGoMailAccounts *accounts;
+              SOGoMailAccount *account;
+              SOGoUserFolder *userFolder;
+              
+              userFolder = [[context activeUser] homeFolderInContext: context];
+              accounts = [userFolder lookupName: @"Mail"  inContext: context  acquire: NO];
+              account = [accounts lookupName: [folders objectAtIndex: 1] inContext: localContext acquire: NO];
+
+              if ([account isKindOfClass: [NSException class]])
+                {
+                  result = [NSException exceptionWithHTTPStatus: 500
+                                                         reason: @"Cannot copy messages to other account."];
+                }
+              else
+                {
+                  NSEnumerator *messages;
+                  NSDictionary *message;
+                  NSData *content;
+                  NSArray *flags;
+
+                  // Fetch messages
+                  result = [client fetchUids: uids parts: [NSArray arrayWithObjects: @"RFC822", @"FLAGS", nil]];
+                  if ([[result objectForKey: @"result"] boolValue])
+                    {
+                      result = [result valueForKey: @"fetch"];
+                      if ([result isKindOfClass: [NSArray class]] && [result count] > 0)
+                        {
+                          // Copy each message to the other account
+                          client = [[account imap4Connection] client];
+                          [[account imap4Connection] selectFolder: imapDestinationFolder];
+                          messages = [result objectEnumerator];
+                          result = nil;
+                          while (result == nil && (message = [messages nextObject]))
+                            {
+                              if ((content = [message valueForKey: @"message"]) != nil)
+                                {
+                                  flags = [message valueForKey: @"flags"];
+                                  result = [client append: content toFolder: imapDestinationFolder withFlags: flags];
+                                  if ([[result objectForKey: @"result"] boolValue])
+                                    result = nil;
+                                  else
+                                    [self logWithFormat: @"ERROR: Can't append message: %@", result];
+                                }
+                            }
+                        }
+                      else
+                        {
+                          [self logWithFormat: @"ERROR: unexpected IMAP4 result (missing 'fetch'): %@", result];
+                          result = [NSException exceptionWithHTTPStatus: 500
+                                                                 reason: @"Unexpected IMAP4 result"];
+                        }
+                    }
+                  else
+                    {
+                      [self logWithFormat: @"ERROR: Can't fetch messages: %@", result];
+                      result = [NSException exceptionWithHTTPStatus: 500
+                                                             reason: @"Can't fetch messages"];
+                    }
+                }
+            }
         }
       else
-        result = [NSException exceptionWithHTTPStatus: 500
-                                               reason: @"Cannot copy messages across different accounts."];
+        result = [NSException exceptionWithName: @"SOGoMailException"
+                                         reason: @"IMAP connection is invalid"
+                                       userInfo: nil];
     }
   else
     result = [NSException exceptionWithHTTPStatus: 500
@@ -739,8 +830,9 @@ static NSString *defaultUserID =  @"anyone";
 - (NSArray *) fetchUIDs: (NSArray *) _uids
 		  parts: (NSArray *) _parts
 {
-  return [[self imap4Connection] fetchUIDs: _uids inURL: [self imap4URL]
-				 parts: _parts];
+  return [[self imap4Connection] fetchUIDs: _uids
+                                     inURL: [self imap4URL]
+                                     parts: _parts];
 }
 
 - (NSArray *) fetchUIDsOfVanishedItems: (uint64_t) modseq
@@ -766,7 +858,7 @@ static NSString *defaultUserID =  @"anyone";
                                 toFolderURL: [self imap4URL]];
   
   return [NSException exceptionWithHTTPStatus: 502 /* Bad Gateway */
-		      reason: [NSString stringWithFormat: @"%@ is not an IMAP4 folder", [self relativeImap4Name]]];
+                                       reason: [NSString stringWithFormat: @"%@ is not an IMAP4 folder", [self relativeImap4Name]]];
 }
 
 - (NSException *) expunge
@@ -938,7 +1030,7 @@ static NSString *defaultUserID =  @"anyone";
 
   if ([self imap4Connection])
     {
-      error = [imap4 createMailbox: [self relativeImap4Name]
+      error = [imap4 createMailbox: [[self relativeImap4Name] stringByEncodingImap4FolderName]
                              atURL: [container imap4URL]];
       if (error)
         rc = NO;
@@ -977,7 +1069,11 @@ static NSString *defaultUserID =  @"anyone";
   NSException *error;
 
   if ([self imap4Connection])
-    error = [imap4 deleteMailboxAtURL: [self imap4URL]];
+    {
+      error = [imap4 deleteMailboxAtURL: [self imap4URL]];
+      if (!error)
+        [[imap4 client] unsubscribe: [[self imap4URL] path]];
+    }
   else
     error = [NSException exceptionWithName: @"SOGoMailException"
                                     reason: @"IMAP connection is invalid"
@@ -1869,6 +1965,234 @@ static NSString *defaultUserID =  @"anyone";
   return date;
 }
 
+- (NSString *) davCollectionTagFromId: (NSString *) theId
+{
+  NSString *tag;
+
+  tag = @"-1";
+
+  if ([self imap4Connection])
+    {
+      NSDictionary *result;
+      unsigned int modseq, uid;
+
+      uid = [theId intValue];
+      result = [[imap4 client] fetchModseqForUid: uid];
+      modseq = [[[[result objectForKey: @"RawResponse"]  objectForKey: @"fetch"] objectForKey: @"modseq"] intValue];
+      
+      if (modseq < 1)
+        modseq = 1;
+
+      tag = [NSString stringWithFormat: @"%d-%d", uid, modseq];
+    }
+
+  return tag;
+}
+
+- (NSString *) davCollectionTag
+{
+  NSString *tag;
+
+  tag = @"-1";
+
+  if ([self imap4Connection])
+    {
+      NSString *folderName;
+      NSDictionary *result;
+
+      folderName = [imap4 imap4FolderNameForURL: [self imap4URL]];
+
+      [[imap4 client] unselect];
+      
+      result = [[imap4 client] select: folderName];
+
+      tag = [NSString stringWithFormat: @"%@-%@", [result objectForKey: @"uidnext"], [result objectForKey: @"highestmodseq"]];
+    }
+
+  return tag;
+}
+
+//
+// FIXME - see below for code refactoring with MAPIStoreMailFolder.
+//
+- (EOQualifier *) _nonDeletedQualifier
+{
+  static EOQualifier *nonDeletedQualifier = nil;
+  EOQualifier *deletedQualifier;
+
+  if (!nonDeletedQualifier)
+    {
+      deletedQualifier
+        = [[EOKeyValueQualifier alloc] 
+                 initWithKey: @"FLAGS"
+            operatorSelector: EOQualifierOperatorContains
+                       value: [NSArray arrayWithObject: @"Deleted"]];
+      nonDeletedQualifier = [[EONotQualifier alloc]
+                              initWithQualifier: deletedQualifier];
+      [deletedQualifier release];
+    }
+
+  return nonDeletedQualifier;
+}
+
+
+
+//
+// Check updated items
+//
+// . UID FETCH 1:* (UID) (CHANGEDSINCE 1)
+// * 1 FETCH (UID 542 MODSEQ (7))
+// * 2 FETCH (UID 553 MODSEQ (14))
+// * 3 FETCH (UID 554 MODSEQ (16))
+// * 4 FETCH (UID 555 MODSEQ (15))
+// * 5 FETCH (UID 559 MODSEQ (17))
+// * 6 FETCH (UID 560 MODSEQ (18))
+// * 7 FETCH (UID 561 MODSEQ (19))
+//
+// SORT + MODSEQ:  http://www.watersprings.org/pub/id/draft-melnikov-condstore-sort-00.txt
+// With date, not modseq
+// . UID SORT (DATE) UTF-8 (NOT DELETED) (SINCE "15-Mar-2014")
+// * SORT 553 542 555 554 601 559 560 561 565 602 603 605 611 610 612 613 614 615 616 617 618 621 619 620 622 623
+//
+// . UID SORT (DATE) UTF-8 ((MODSEQ 64) (NOT DELETED)) (SINCE "15-Mar-2014")
+// * SORT 623 624 (MODSEQ 65)
+// . OK Completed (2 msgs in 0.000 secs)
+//
+// ".. the server MUST also append (to the end of the untagged SORT response) the highest mod-sequence for all messages being returned."
+//                                                    
+// To get the modseq of a specific message:
+//
+// . UID FETCH 124569:124569 (UID MODSEQ)
+// * 4900 FETCH (UID 124569 MODSEQ (2))
+//
+//
+// To get deleted messages
+//
+// . UID FETCH 1:* (UID) (CHANGEDSINCE 1 VANISHED)
+// * VANISHED (EARLIER) 1:541,543:552,556:558,562:564,566:600,604,606:609
+// * 1 FETCH (UID 542 MODSEQ (7))
+// * 2 FETCH (UID 553 MODSEQ (14))
+// * 3 FETCH (UID 554 MODSEQ (16))
+// * 4 FETCH (UID 555 MODSEQ (15))
+// * 5 FETCH (UID 559 MODSEQ (17))
+// * 6 FETCH (UID 560 MODSEQ (18))
+// * 7 FETCH (UID 561 MODSEQ (19))
+
+//
+// fetchUIDsOfVanishedItems ..
+//
+// . uid fetch 1:* (FLAGS) (changedsince 176 vanished)
+// * VANISHED (EARLIER) 36
+//
+// 
+// FIXME: refactor MAPIStoreMailFolder.m - synchroniseCache to use this method
+//
+- (NSArray *) syncTokenFieldsWithProperties: (NSArray *) theProperties
+                          matchingSyncToken: (NSString *) theSyncToken
+                                   fromDate: (NSCalendarDate *) theStartDate
+{
+  EOQualifier *searchQualifier;
+  NSMutableArray *allTokens;
+  NSArray *a, *uids;
+  NSDictionary *d;
+  id fetchResults;
+
+  int uidnext, highestmodseq, i; 
+
+  allTokens = [NSMutableArray array];
+
+  if ([theSyncToken isEqualToString: @"-1"])
+    {
+      uidnext = highestmodseq = 0;
+    }
+  else
+    {
+      a = [theSyncToken componentsSeparatedByString: @"-"];
+      uidnext = [[a objectAtIndex: 0] intValue];
+      highestmodseq = [[a objectAtIndex: 1] intValue];
+    }
+  
+  // We first make sure QRESYNC is enabled
+  [[self imap4Connection] enableExtensions: [NSArray arrayWithObject: @"QRESYNC"]];
+   
+
+  // We fetch new messages and modified messages
+  if (highestmodseq)
+    {    
+      EOKeyValueQualifier *kvQualifier;
+      NSNumber *nextModseq;
+
+      nextModseq = [NSNumber numberWithUnsignedLongLong: highestmodseq];
+      kvQualifier = [[EOKeyValueQualifier alloc]
+                                initWithKey: @"modseq"
+                           operatorSelector: EOQualifierOperatorGreaterThanOrEqualTo
+                                      value: nextModseq];
+      searchQualifier = [[EOAndQualifier alloc]
+                          initWithQualifiers:
+                            kvQualifier, nil];
+      [kvQualifier release];
+      [searchQualifier autorelease];
+    }
+  else
+    {
+      searchQualifier = nil;
+    }
+
+  if (theStartDate)
+    {
+      EOQualifier *sinceDateQualifier;
+      
+      sinceDateQualifier = [EOQualifier qualifierWithQualifierFormat:
+                                          @"(DATE >= %@)", theStartDate];
+      searchQualifier = [[EOAndQualifier alloc] initWithQualifiers: sinceDateQualifier, searchQualifier,
+                                                nil];
+      [searchQualifier autorelease];
+    }
+  
+
+  // we fetch modified or added uids
+  uids = [self fetchUIDsMatchingQualifier: searchQualifier
+                             sortOrdering: nil];
+
+  fetchResults = [(NSDictionary *)[self fetchUIDs: uids
+                                            parts: [NSArray arrayWithObjects: @"modseq", @"flags", nil]]
+                     objectForKey: @"fetch"];
+  
+  /* NOTE: we sort items manually because Cyrus does not properly sort
+     entries with a MODSEQ of 0 */
+  fetchResults
+    = [fetchResults sortedArrayUsingFunction: _compareFetchResultsByMODSEQ
+                                     context: NULL];
+
+  for (i = 0; i < [fetchResults count]; i++)
+    { 
+      d = [NSDictionary dictionaryWithObject: ([[[fetchResults objectAtIndex: i] objectForKey: @"flags"] containsObject: @"deleted"]) ? [NSNull null] : [[fetchResults objectAtIndex: i] objectForKey: @"modseq"]
+                                      forKey: [[[fetchResults objectAtIndex: i] objectForKey: @"uid"] stringValue]];
+      [allTokens addObject: d];
+    }
+  
+
+  // We fetch deleted ones
+  if (highestmodseq == 0)
+    highestmodseq = 1;
+
+  if (highestmodseq > 0)
+    {
+      id uid;
+
+      uids = [self fetchUIDsOfVanishedItems: highestmodseq];
+      
+      for (i = 0; i < [uids count]; i++)
+        {
+          uid = [[uids objectAtIndex: i] stringValue];
+          d = [NSDictionary dictionaryWithObject: [NSNull null]  forKey: uid];
+          [allTokens addObject: d];
+        }
+    }
+
+  return allTokens;
+}
+
 @end /* SOGoMailFolder */
 
 @implementation SOGoSpecialMailFolder
