@@ -1,7 +1,7 @@
 /* NSString+Crypto.h - this file is part of SOGo
  *
  * Copyright (C) 2012 Nicolas Höft
- * Copyright (C) 2012 Inverse inc.
+ * Copyright (C) 2012-2015 Inverse inc.
  *
  * Author: Nicolas Höft
  *         Inverse inc.
@@ -39,13 +39,12 @@ typedef enum {
 
 @interface NSString (SOGoCryptoExtension)
 
-
 - (BOOL) isEqualToCrypted: (NSString *) cryptedPassword
-         withDefaultScheme: (NSString *) theScheme;
+        withDefaultScheme: (NSString *) theScheme;
 
 - (NSString *) asCryptedPassUsingScheme: (NSString *) passwordScheme
                                withSalt: (NSData *) theSalt
-                               andEncoding: (keyEncoding) encoding;
+                            andEncoding: (keyEncoding) encoding;
 
 // this method uses the default encoding (base64, plain, hex)
 // and generates a salt when necessary
@@ -56,6 +55,9 @@ typedef enum {
 - (NSString *) asSHA1String;
 - (NSString *) asMD5String;
 
+- (NSString *) asNTHash;
+- (NSString *) asLMHash;
+
 + (NSArray *) getDefaultEncodingForScheme: (NSString *) passwordScheme;
 
 @end
