@@ -1,8 +1,6 @@
 /* SOGoDomainDefaults.h - this file is part of SOGo
  *
- * Copyright (C) 2009-2013 Inverse inc.
- *
- * Author: Wolfgang Sourdeau <wsourdeau@inverse.ca>
+ * Copyright (C) 2009-2015 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -51,6 +49,7 @@
 - (BOOL) forceExternalLoginWithEmail;
 - (BOOL) sieveScriptsEnabled;
 - (BOOL) forwardEnabled;
+- (int) forwardConstraints;
 - (BOOL) vacationEnabled;
 - (NSString *) mailingMechanism;
 - (NSString *) smtpServer;
@@ -66,7 +65,7 @@
 - (BOOL) foldersSendEMailNotifications;
 - (NSArray *) calendarDefaultRoles;
 - (NSArray *) contactsDefaultRoles;
-- (NSArray *) mailPollingIntervals;
+- (NSArray *) refreshViewIntervals;
 - (NSString *) subscriptionFolderFormat;
 
 - (NSString *) calendarDefaultCategoryColor;
