@@ -1,8 +1,6 @@
 /* SOGoDomainDefaults.m - this file is part of SOGo
  *
- * Copyright (C) 2009-2014 Inverse inc.
- *
- * Author: Wolfgang Sourdeau <wsourdeau@inverse.ca>
+ * Copyright (C) 2009-2015 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -203,6 +201,15 @@
   return [self boolForKey: @"SOGoForwardEnabled"];
 }
 
+- (int) forwardConstraints
+{
+  unsigned int v;
+
+  v =  [self integerForKey: @"SOGoForwardConstraints"];
+  
+  return (v > 2 ? 0 : v);
+}
+
 - (BOOL) vacationEnabled
 {
   return [self boolForKey: @"SOGoVacationEnabled"];
@@ -225,9 +232,9 @@
   return [mailingMechanism lowercaseString];
 }
 
-- (NSArray *) mailPollingIntervals
+- (NSArray *) refreshViewIntervals
 {
-  return [self arrayForKey: @"SOGoMailPollingIntervals"];
+  return [self arrayForKey: @"SOGoRefreshViewIntervals"];
 }
 
 - (NSString *) smtpServer
