@@ -1,8 +1,6 @@
 /* SOGoGroup.m - this file is part of SOGo
  *
- * Copyright (C) 2009-2012 Inverse inc.
- *
- * Author: Ludovic Marcotte <lmarcotte@inverse.ca>
+ * Copyright (C) 2009-2014 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -139,6 +137,8 @@
   NGLdapEntry *entry;
   NSObject <SOGoSource> *source;
   id o;
+  NSEnumerator *gclasses;
+  NSString *gclass;
   
   int i;
 
@@ -196,12 +196,11 @@
             }
 	}
 
-      // Found a group, let's return it.
-      if ([classes containsObject: @"group"] ||
-	  [classes containsObject: @"groupofnames"] ||
-	  [classes containsObject: @"groupofuniquenames"] ||
-	  [classes containsObject: @"posixgroup"])
-	{
+        gclasses = [[source groupObjectClasses] objectEnumerator];
+        while (gclass = [gclasses nextObject])
+          if ([classes containsObject: gclass])
+           {
+          // Found a group, let's return it.
 	  o = [[self alloc] initWithIdentifier: theValue
 			                domain: domain
                                         source: source
