@@ -1,6 +1,6 @@
 /* SOGoUserDefaults.h - this file is part of SOGo
  *
- * Copyright (C) 2011-2013 Inverse inc.
+ * Copyright (C) 2011-2014 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -117,8 +117,8 @@ extern NSString *SOGoWeekStartFirstFullWeek;
 - (void) setSelectedAddressBook: (NSString *) newValue;
 - (NSString *) selectedAddressBook;
 
-- (void) setMailMessageCheck: (NSString *) newValue;
-- (NSString *) mailMessageCheck;
+- (void) setRefreshViewCheck: (NSString *) newValue;
+- (NSString *) refreshViewCheck;
 
 - (void) setMailComposeMessageType: (NSString *) newValue;
 - (NSString *) mailComposeMessageType;
@@ -126,6 +126,9 @@ extern NSString *SOGoWeekStartFirstFullWeek;
 - (void) setMailDisplayRemoteInlineImages: (NSString *) newValue;
 - (NSString *) mailDisplayRemoteInlineImages;
 
+- (void) setMailAutoSave: (NSString *) newValue;
+- (NSString *) mailAutoSave;
+
 - (void) setMailMessageForwarding: (NSString *) newValue;
 - (NSString *) mailMessageForwarding;
 
