@@ -1,8 +1,6 @@
 /* SOGoUserSettings.h - this file is part of SOGo
  *
- * Copyright (C) 2009 Inverse inc.
- *
- * Author: Wolfgang Sourdeau <wsourdeau@inverse.ca>
+ * Copyright (C) 2009-2014 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,6 +24,7 @@
 #import "SOGoDefaultsSource.h"
 
 @class NSArray;
+@class NSMutableDictionary;
 @class NSString;
 
 @interface SOGoUserSettings : SOGoDefaultsSource
