@@ -401,7 +401,17 @@ class DAVCalendarAclTest(DAVAclTest):
         else:
             expStatus = 207
         privileges = self._currentUserPrivilegeSet(self.resource, expStatus)
-        self._comparePrivilegeSets(expectedPrivileges, privileges)
+
+        # When comparing privileges on DAV collection, we remove all 'default'
+        # privileges on the collection.
+        privileges_set = set(privileges);
+        for x in ("public", "private", "confidential"):
+            privileges_set.discard("{urn:inverse:params:xml:ns:inverse-dav}viewwhole-%s-records" % x)
+            privileges_set.discard("{urn:inverse:params:xml:ns:inverse-dav}viewdant-%s-records" % x)
+            privileges_set.discard("{urn:inverse:params:xml:ns:inverse-dav}modify-%s-records" % x)
+            privileges_set.discard("{urn:inverse:params:xml:ns:inverse-dav}respondto-%s-records" %x)
+
+        self._comparePrivilegeSets(expectedPrivileges, list(privileges_set))
 
     def _testEventDAVAcl(self, event_class, right, error_code):
         icsClass = self.classToICSClass[event_class].lower()
@@ -673,12 +683,12 @@ class DAVCalendarAclTest(DAVAclTest):
             if right == "r":
                 exp_event = event_template % {"class": icsClass,
                                               "filename": filename,
-                                              "organizer_line": "ORGANIZER:mailto:nobody@somewhere.com\n",
+                                              "organizer_line": "ORGANIZER;CN=nobody@somewhere.com:mailto:nobody@somewhere.com\n",
                                               "attendee_line": att_line}
             else:
                 exp_event = event_template % {"class": icsClass,
                                               "filename": filename,
-                                              "organizer_line": "ORGANIZER:mailto:someone@nowhere.com\n",
+                                              "organizer_line": "ORGANIZER;CN=someone@nowhere.com:mailto:someone@nowhere.com\n",
                                               "attendee_line": att_line}
             event = self._getEvent(event_class, True)
             ics_diff = utilities.ics_compare(exp_event, event)
