@@ -113,8 +113,7 @@
 
 - (NSString *) doctype
 {
-  return (@"<?xml version=\"1.0\" encoding=\"utf-8\"?>\n"
-          @"<!DOCTYPE html"
+  return (@"<!DOCTYPE html"
           @" PUBLIC \"-//W3C//DTD XHTML 1.1//EN\""
           @" \"http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd\">");
 }
