@@ -86,6 +86,9 @@ _xmlCharsetForCharset (NSString *charset)
     { @"windows-1251", XML_CHAR_ENCODING_ERROR}, // unsupported, will trigger windows-1251 -> utf8 conversion
     { @"windows-1257", XML_CHAR_ENCODING_ERROR}, // unsupported, will trigger windows-1257 -> utf8 conversion
     { @"gb2312", XML_CHAR_ENCODING_ERROR},       // unsupported, will trigger gb2312 -> utf8 conversion
+    { @"gbk", XML_CHAR_ENCODING_ERROR},          // unsupported, will trigger gb2312 -> utf8 conversion
+    { @"gb18030", XML_CHAR_ENCODING_ERROR},      // unsupported, will trigger gb2312 -> utf8 conversion
+    { @"big5", XML_CHAR_ENCODING_ERROR},         // unsupported, will trigger gb2312 -> utf8 conversion
     { @"euc-jp", XML_CHAR_ENCODING_EUC_JP}};
   unsigned count;
   xmlCharEncoding encoding;
@@ -143,34 +146,34 @@ static NSData* _sanitizeContent(NSData *theData)
   const char *bytes;
   char *buf;
   int i, j, len;
-  BOOL found_delimiter;
+  BOOL found_delimiter, in_meta;
 
   d = [NSMutableData dataWithData: theData];
   bytes = [d bytes];
   len = [d length];
   i = 0;
 
+  in_meta = NO;
+
   while (i < len)
     {
-      // We check if we see </head> in which case, we don't do any kind
-      // of substitution there after.
-      if (i < len-6)
+      // We check if we see <meta ...> in which case, we substitute de charset= stuff.
+      if (i < len-5)
 	{
 	  if ((*bytes == '<') &&
-	      (*(bytes+1) == '/') &&
-	      (*(bytes+2) == 'h' || *(bytes+2) == 'H') &&
-	      (*(bytes+3) == 'e' || *(bytes+3) == 'E') &&
-	      (*(bytes+4) == 'a' || *(bytes+4) == 'A') &&
-	      (*(bytes+5) == 'd' || *(bytes+5) == 'D') &&
-	      (*(bytes+6) == '>'))
-            break;
+	      (*(bytes+1) == 'm' || *(bytes+2) == 'M') &&
+	      (*(bytes+2) == 'e' || *(bytes+3) == 'E') &&
+	      (*(bytes+3) == 't' || *(bytes+4) == 'T') &&
+	      (*(bytes+4) == 'a' || *(bytes+5) == 'A') &&
+	      (*(bytes+5) == ' '))
+            in_meta = YES;
 	}
       
       // We search for something like :
       // 
       // <meta http-equiv="Content-Type" content="text/html; charset=Windows-1252">
       //
-      if (i < len-9)
+      if (in_meta && i < len-9)
 	{
 	  if ((*bytes == 'c' || *bytes == 'C') &&
 	      (*(bytes+1) == 'h' || *(bytes+1) == 'H') &&
@@ -192,16 +195,18 @@ static NSData* _sanitizeContent(NSData *theData)
 		  // We haven't found anything, let's return the data untouched
 		  if ((i+j) >= len)
                     {
-                      found_delimiter = NO;
+                      in_meta = found_delimiter = NO;
                       break;
                     }
 		}
 
               if (found_delimiter)
-                [d replaceBytesInRange: NSMakeRange(i, j)
-                             withBytes: NULL
-                                length: 0];
-	      break;
+                {
+                  [d replaceBytesInRange: NSMakeRange(i, j)
+                               withBytes: NULL
+                                  length: 0];
+                  in_meta = found_delimiter = NO;
+                }
 	    }
 	}
 
@@ -267,7 +272,7 @@ static NSData* _sanitizeContent(NSData *theData)
                       if ([tag caseInsensitiveCompare: found_tag] == NSOrderedSame)
                         {
                           // Remove the leading slash
-                          NSLog(@"Found void tag with invalid leading slash: </%@>", found_tag);
+                          //NSLog(@"Found void tag with invalid leading slash: </%@>", found_tag);
                           i--;
                           [d replaceBytesInRange: NSMakeRange(i, 1)
                                        withBytes: NULL
@@ -416,9 +421,9 @@ static NSData* _sanitizeContent(NSData *theData)
 }
 
 - (void) _appendStyle: (unichar *) _chars
-               length: (int) _len
+               length: (NSUInteger) _len
 {
-  unsigned int count, length;
+  NSUInteger count, length;
   unichar *start, *currentChar;
 
   start = _chars;
@@ -450,6 +455,10 @@ static NSData* _sanitizeContent(NSData *theData)
             {
               if (*currentChar == '{')
                 inCSSDeclaration = YES;
+              if (*currentChar == '}')
+                // CSS syntax error: ending declaration character while not in a CSS declaration.
+                // Ignore eveything from last CSS declaration.
+                start = currentChar + 1;
               else if (*currentChar == ',')
                 hasEmbeddedCSS = NO;
               else if (!hasEmbeddedCSS)
@@ -685,7 +694,7 @@ static NSData* _sanitizeContent(NSData *theData)
 }
 
 - (void) characters: (unichar *) _chars
-             length: (int) _len
+             length: (NSUInteger) _len
 {
   showWhoWeAre();
   if (!ignoredContent)
@@ -709,7 +718,7 @@ static NSData* _sanitizeContent(NSData *theData)
 }
 
 - (void) ignorableWhitespace: (unichar *) _chars
-                      length: (int) _len
+                      length: (NSUInteger) _len
 {
   showWhoWeAre();
 }
@@ -732,7 +741,7 @@ static NSData* _sanitizeContent(NSData *theData)
 
 /* SaxLexicalHandler */
 - (void) comment: (unichar *) _chars
-          length: (int) _len
+          length: (NSUInteger) _len
 {
   showWhoWeAre();
   if (inStyle)
