@@ -1,8 +1,6 @@
 /* UIxMailAccountActions.h - this file is part of SOGo
  *
- * Copyright (C) 2007-2011 Inverse inc.
- *
- * Author: Wolfgang Sourdeau <wsourdeau@inverse.ca>
+ * Copyright (C) 2007-2013 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,12 +27,6 @@
 
 @interface UIxMailAccountActions : WODirectAction
 {
-  NSString *inboxFolderName;
-  NSString *draftsFolderName;
-  NSString *sentFolderName;
-  NSString *trashFolderName;
-  NSString *otherUsersFolderName;
-  NSString *sharedFoldersName;
 }
 
 - (WOResponse *) listMailboxesAction;
