@@ -1,6 +1,6 @@
 /* UIxFilterEditor.m - this file is part of SOGo
  *
- * Copyright (C) 2010-2013 Inverse inc.
+ * Copyright (C) 2010-2014 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,6 +28,7 @@
 #import <SOGo/NSString+Utilities.h>
 #import <SOGo/SOGoUser.h>
 #import <SOGo/SOGoUserDefaults.h>
+#import <SOGo/SOGoSystemDefaults.h>
 
 #import <SOGoUI/UIxComponent.h>
 
@@ -112,4 +113,9 @@
   return [labels jsonRepresentation];
 }
 
+- (NSString *) sieveFolderEncoding
+{
+  return [[SOGoSystemDefaults sharedSystemDefaults] sieveFolderEncoding];
+}
+
 @end
