@@ -1,9 +1,6 @@
 /* UIxCalListingActions.m - this file is part of SOGo
  *
- * Copyright (C) 2006-2011 Inverse inc.
- *
- * Author: Wolfgang Sourdeau <wsourdeau@inverse.ca>
- *         Francis Lachapelle <flachapelle@inverse.ca>
+ * Copyright (C) 2006-2014 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -102,7 +99,7 @@ static NSArray *tasksFields = nil;
                    @"c_status", @"c_title", @"c_enddate",
                    @"c_classification", @"c_location", @"c_category",
                    @"editable", @"erasable",
-                   @"c_priority", nil];
+                   @"c_priority", @"c_owner", @"c_recurrence_id", @"isException", nil];
     [tasksFields retain];
   }
 }
@@ -121,6 +118,7 @@ static NSArray *tasksFields = nil;
     ASSIGN (dateFormatter, [user dateFormatterInContext: context]);
     ASSIGN (userTimeZone, [[user userDefaults] timeZone]);
     dayBasedView = NO;
+    currentView = nil;
   }
   
   return self;
@@ -239,6 +237,7 @@ static NSArray *tasksFields = nil;
       endDate = nil;
     
     param = [request formValueForKey: @"view"];
+    currentView = param;
     dayBasedView = ![param isEqualToString: @"monthview"];
   }
 }
@@ -332,155 +331,163 @@ static NSArray *tasksFields = nil;
   
   folders = [[clientObject subFolders] objectEnumerator];
   while ((currentFolder = [folders nextObject]))
-  {
-    if ([currentFolder isActive])
     {
-      folderIsRemote = [currentFolder isKindOfClass: [SOGoWebAppointmentFolder class]];
+      if ([currentFolder isActive])
+        {
+          folderIsRemote = [currentFolder isKindOfClass: [SOGoWebAppointmentFolder class]];
       
-      if ([criteria isEqualToString:@"title_Category_Location"])
-        currentInfos = [[currentFolder fetchCoreInfosFrom: startDate
-                                                       to: endDate
-                                                    title: value
-                                                component: component
-                                        additionalFilters: criteria] objectEnumerator];
-      
-      else if ([criteria isEqualToString:@"entireContent"])
-      {
-        // First research : Through the quick table inside the location, category and title columns
-        quickInfos = [currentFolder fetchCoreInfosFrom: startDate
-                                                    to: endDate
-                                                 title: value
-                                             component: component
-                                     additionalFilters: criteria];
-        
-        // Save the c_name in another array to compare with
-        if ([quickInfos count] > 0)
-        {
-          quickInfosFlag = YES;
-          quickInfosName = [NSMutableArray arrayWithCapacity:[quickInfos count]];
-          for (i = 0; i < [quickInfos count]; i++)
-            [quickInfosName addObject:[[quickInfos objectAtIndex:i] objectForKey:@"c_name"]];
-        }
-        
-        // Second research : Every objects except for those already in the quickInfos array
-        allInfos = [currentFolder fetchCoreInfosFrom: startDate
-                                                  to: endDate
-                                               title: nil
-                                           component: component];
-        if (quickInfosFlag == YES)
-        {
-          for (i = ([allInfos count] - 1); i >= 0 ; i--) {
-            if([quickInfosName containsObject:[[allInfos objectAtIndex:i] objectForKey:@"c_name"]])
-              [allInfos removeObjectAtIndex:i];
-          }
-        }
-        
-        
-        for (i = 0; i < [allInfos count]; i++)
-        {
-          iCalString = [[allInfos objectAtIndex:i] objectForKey:@"c_content"];
-          calendar = [iCalCalendar parseSingleFromSource: iCalString];
-          master = [calendar firstChildWithTag:component];
-          if (master) {
-            if ([[master comment] length] > 0)
+          //
+          // criteria can have the following values: "title", "title_Category_Location" and "entireContent"
+          //
+          if ([criteria isEqualToString:@"title_Category_Location"])
             {
-              match = [[master comment] rangeOfString:value options:NSCaseInsensitiveSearch];
-              if (match.length > 0) {
-                [quickInfos addObject:[allInfos objectAtIndex:i]];
-              }
+              currentInfos = [[currentFolder fetchCoreInfosFrom: startDate
+                                                             to: endDate
+                                                          title: value
+                                                      component: component
+                                              additionalFilters: criteria] objectEnumerator];
             }
-          }
-        }
+          else if ([criteria isEqualToString:@"entireContent"])
+            {
+              // First search : Through the quick table inside the location, category and title columns
+              quickInfos = [currentFolder fetchCoreInfosFrom: startDate
+                                                          to: endDate
+                                                       title: value
+                                                   component: component
+                                           additionalFilters: criteria];
         
-      currentInfos = [quickInfos objectEnumerator];
-      }
-      
+              // Save the c_name in another array to compare with
+              if ([quickInfos count] > 0)
+                {
+                  quickInfosFlag = YES;
+                  quickInfosName = [NSMutableArray arrayWithCapacity:[quickInfos count]];
+                  for (i = 0; i < [quickInfos count]; i++)
+                    [quickInfosName addObject:[[quickInfos objectAtIndex:i] objectForKey:@"c_name"]];
+                }
+        
+              // Second research : Every objects except for those already in the quickInfos array
+              allInfos = [currentFolder fetchCoreInfosFrom: startDate
+                                                        to: endDate
+                                                     title: nil
+                                                 component: component];
+              if (quickInfosFlag == YES)
+                {
+                  for (i = ([allInfos count] - 1); i >= 0 ; i--) {
+                    if([quickInfosName containsObject:[[allInfos objectAtIndex:i] objectForKey:@"c_name"]])
+                      [allInfos removeObjectAtIndex:i];
+                  }
+                }
+        
+        
+              for (i = 0; i < [allInfos count]; i++)
+                {
+                  iCalString = [[allInfos objectAtIndex:i] objectForKey:@"c_content"];
+                  calendar = [iCalCalendar parseSingleFromSource: iCalString];
+                  master = [calendar firstChildWithTag:component];
+                  if (master) {
+                    if ([[master comment] length] > 0)
+                      {
+                        match = [[master comment] rangeOfString:value options:NSCaseInsensitiveSearch];
+                        if (match.length > 0) {
+                          [quickInfos addObject:[allInfos objectAtIndex:i]];
+                        }
+                      }
+                  }
+                }
+        
+              currentInfos = [quickInfos objectEnumerator];
+            }
+          else
+            {
+              id foo;
 
-      else
-        currentInfos = [[currentFolder fetchCoreInfosFrom: startDate
-                                                       to: endDate
-                                                    title: value
-                                                component: component] objectEnumerator];
+              foo = [currentFolder fetchCoreInfosFrom: startDate
+                                                             to: endDate
+                                                          title: value
+                                            component: component];
+              currentInfos = [foo objectEnumerator];
+            }
       
-      owner = [currentFolder ownerInContext: context];
-      ownerUser = [SOGoUser userWithLogin: owner];
-      isErasable = ([owner isEqualToString: userLogin] || [[currentFolder aclsForUser: userLogin] containsObject: SOGoRole_ObjectEraser]);
+          owner = [currentFolder ownerInContext: context];
+          ownerUser = [SOGoUser userWithLogin: owner];
+          isErasable = ([owner isEqualToString: userLogin] || [[currentFolder aclsForUser: userLogin] containsObject: SOGoRole_ObjectEraser]);
       
-      while ((newInfo = [currentInfos nextObject]))
-      {
-        if ([fields containsObject: @"editable"])
-        {
-          if (folderIsRemote)
-            // .ics subscriptions are not editable
-            [newInfo setObject: [NSNumber numberWithInt: 0]
-                        forKey: @"editable"];
-          else
-          {
-            // Identifies whether the active user can edit the event.
-            role = [currentFolder roleForComponentsWithAccessClass:[[newInfo objectForKey: @"c_classification"] intValue]
-                                                           forUser: userLogin];
-            if ([role isEqualToString: @"ComponentModifier"] || [role length] == 0)
-              [newInfo setObject: [NSNumber numberWithInt: 1]
-                          forKey: @"editable"];
-            else
-              [newInfo setObject: [NSNumber numberWithInt: 0]
-                          forKey: @"editable"];
-          }
-        }
+          while ((newInfo = [currentInfos nextObject]))
+            {
+              if ([fields containsObject: @"editable"])
+                {
+                  if (folderIsRemote)
+                    // .ics subscriptions are not editable
+                    [newInfo setObject: [NSNumber numberWithInt: 0]
+                                forKey: @"editable"];
+                  else
+                    {
+                      // Identifies whether the active user can edit the event.
+                      role = [currentFolder roleForComponentsWithAccessClass:[[newInfo objectForKey: @"c_classification"] intValue]
+                                                                     forUser: userLogin];
+                      if ([role isEqualToString: @"ComponentModifier"] || [role length] == 0)
+                        [newInfo setObject: [NSNumber numberWithInt: 1]
+                                    forKey: @"editable"];
+                      else
+                        [newInfo setObject: [NSNumber numberWithInt: 0]
+                                    forKey: @"editable"];
+                    }
+                }
 	      if ([fields containsObject: @"ownerIsOrganizer"])
-        {
-          // Identifies whether the active user is the organizer
-          // of this event.
-          NSString *c_orgmail;
-          c_orgmail = [newInfo objectForKey: @"c_orgmail"];
+                {
+                  // Identifies whether the active user is the organizer
+                  // of this event.
+                  NSString *c_orgmail;
+                  c_orgmail = [newInfo objectForKey: @"c_orgmail"];
           
-          if ([c_orgmail isKindOfClass: [NSString class]] && [ownerUser hasEmail: c_orgmail])
-            [newInfo setObject: [NSNumber numberWithInt: 1]
-                        forKey: @"ownerIsOrganizer"];
-          else
-            [newInfo setObject: [NSNumber numberWithInt: 0]
-                        forKey: @"ownerIsOrganizer"];
-        }
+                  if ([c_orgmail isKindOfClass: [NSString class]] && [ownerUser hasEmail: c_orgmail])
+                    [newInfo setObject: [NSNumber numberWithInt: 1]
+                                forKey: @"ownerIsOrganizer"];
+                  else
+                    [newInfo setObject: [NSNumber numberWithInt: 0]
+                                forKey: @"ownerIsOrganizer"];
+                }
 	      if (isErasable)
-          [newInfo setObject: [NSNumber numberWithInt: 1]
-                      forKey: @"erasable"];
+                [newInfo setObject: [NSNumber numberWithInt: 1]
+                            forKey: @"erasable"];
 	      else
-          [newInfo setObject: [NSNumber numberWithInt: 0]
-                      forKey: @"erasable"];
+                [newInfo setObject: [NSNumber numberWithInt: 0]
+                            forKey: @"erasable"];
         
 	      [newInfo setObject: [currentFolder nameInContainer]
-                    forKey: @"c_folder"];
-        [newInfo setObject: [currentFolder ownerInContext: context]
-                    forKey: @"c_owner"];
-        calendarName = [currentFolder displayName];
-        if (calendarName == nil)
-          calendarName = @"";
-        [newInfo setObject: calendarName
-                    forKey: @"calendarName"];
-        if (![[newInfo objectForKey: @"c_title"] length])
-          [self _fixComponentTitle: newInfo withType: component];
+                          forKey: @"c_folder"];
+              [newInfo setObject: [currentFolder ownerInContext: context]
+                          forKey: @"c_owner"];
+              calendarName = [currentFolder displayName];
+              if (calendarName == nil)
+                calendarName = @"";
+              [newInfo setObject: calendarName
+                          forKey: @"calendarName"];
+              if (![[newInfo objectForKey: @"c_title"] length])
+                [self _fixComponentTitle: newInfo withType: component];
+
 	      // Possible improvement: only call _fixDates if event is recurrent
 	      // or the view range span a daylight saving time change
-        [self _fixDates: newInfo];
-        newInfoForComponent = [NSMutableArray arrayWithArray: [newInfo objectsForKeys: fields
-                                                                       notFoundMarker: marker]];
-        // Escape HTML
-        count = [newInfoForComponent count];
-        for (i = 0; i < count; i++)
-        {
-          currentInfo = [newInfoForComponent objectAtIndex: i];
-          if ([currentInfo respondsToSelector: @selector (stringByEscapingHTMLString)])
-            [newInfoForComponent replaceObjectAtIndex: i withObject: [currentInfo stringByEscapingHTMLString]];
+              [self _fixDates: newInfo];
+              newInfoForComponent = [NSMutableArray arrayWithArray: [newInfo objectsForKeys: fields
+                                                                             notFoundMarker: marker]];
+              // Escape HTML
+              count = [newInfoForComponent count];
+              for (i = 0; i < count; i++)
+                {
+                  currentInfo = [newInfoForComponent objectAtIndex: i];
+                  if ([currentInfo respondsToSelector: @selector (stringByEscapingHTMLString)])
+                    [newInfoForComponent replaceObjectAtIndex: i withObject: [currentInfo stringByEscapingHTMLString]];
+                }
+              [infos addObject: newInfoForComponent];
+            }
         }
-        [infos addObject: newInfoForComponent];
-      }
     }
-  }
   
   return infos;
 }
 
-- (WOResponse *) _responseWithData: (NSArray *) data
+- (WOResponse *) _responseWithData: (id) data
 {
   WOResponse *response;
   
@@ -521,7 +528,6 @@ static NSArray *tasksFields = nil;
 // - for ALL subscribed and ACTIVE calendars
 //  - returns alarms that will occur in the next 48 hours or the non-triggered alarms
 //    for non-completed events
-//  - recurring events are currently ignored
 //
 - (WOResponse *) alarmsListAction
 {
@@ -540,33 +546,28 @@ static NSArray *tasksFields = nil;
   
   folders = [[clientObject subFolders] objectEnumerator];
   while ((currentFolder = [folders nextObject]))
-  {
-    if ([currentFolder isActive] && [currentFolder showCalendarAlarms])
     {
-      NSDictionary *entry;
-      NSArray *alarms;
-      BOOL isCycle;
-      int i;
-      
-      alarms = [currentFolder fetchAlarmInfosFrom: [NSNumber numberWithInt: browserTime]
-                                               to: [NSNumber numberWithInt: laterTime]];
-      
-      for (i = 0; i < [alarms count]; i++)
-	    {
-	      entry = [alarms objectAtIndex: i];
-	      isCycle = [[entry objectForKey: @"c_iscycle"] boolValue];
-	      
-	      if (!isCycle)
+      if ([currentFolder isActive] && [currentFolder showCalendarAlarms])
         {
-          [allAlarms addObject: [NSArray arrayWithObjects:
-                                 [currentFolder nameInContainer],
-                                 [entry objectForKey: @"c_name"],
-                                 [entry objectForKey: @"c_nextalarm"],
-                                 nil]];
+          NSDictionary *entry;
+          NSArray *alarms;
+          int i;
+          
+          alarms = [currentFolder fetchAlarmInfosFrom: [NSNumber numberWithInt: browserTime]
+                                                   to: [NSNumber numberWithInt: laterTime]];
+          
+          for (i = 0; i < [alarms count]; i++)
+            {
+              entry = [alarms objectAtIndex: i];
+              
+              [allAlarms addObject: [NSArray arrayWithObjects:
+                                               [currentFolder nameInContainer],
+                                          [entry objectForKey: @"c_name"],
+                                          [entry objectForKey: @"c_nextalarm"],
+                                             nil]];
+            }
         }
-	    }
     }
-  }
   
   
   response = [self responseWithStatus: 200];
@@ -1044,47 +1045,113 @@ _computeBlocksPosition (NSArray *blocks)
   /* ... _computeBlocksMultiplier() ... */
 }
 
+- (NSArray *) _selectedCalendars
+{
+  SOGoAppointmentFolders *co;
+  SOGoAppointmentFolder *folder;
+  NSMutableArray *selectedCalendars;
+  NSArray *folders;
+  NSString *fUID;
+  NSNumber *isActive;
+  unsigned int count, foldersCount;
+  int max=0, i;
+  
+  co = [self clientObject];
+  folders = [co subFolders];
+  foldersCount = [folders count];
+  selectedCalendars = [[NSMutableArray alloc] initWithCapacity: foldersCount];
+  for (count = 0; count < foldersCount; count++)
+  {
+    folder = [folders objectAtIndex: count];
+    isActive = [NSNumber numberWithBool: [folder isActive]];
+    if ([isActive intValue] != 0) {
+      fUID = [folder nameInContainer];
+      [selectedCalendars addObject: fUID];
+    }
+  }
+  return selectedCalendars;
+}
+
 - (WOResponse *) eventsBlocksAction
 {
   int count, max;
   NSArray *events, *event, *eventsBlocks;
-  NSMutableArray *allDayBlocks, *blocks, *currentDay;
+  NSMutableArray *allDayBlocks, *blocks, *currentDay, *calendars, *eventsByCalendars, *eventsForCalendar;
   NSNumber *eventNbr;
   BOOL isAllDay;
+  int i, j;
   
   [self _setupContext];
   
-  [self _prepareEventBlocks: &blocks withAllDays: &allDayBlocks];
-  events = [self _fetchFields: eventsFields
-           forComponentOfType: @"vevent"];
-  eventsBlocks = [NSArray arrayWithObjects: events, allDayBlocks, blocks, nil];
-  max = [events count];
-  for (count = 0; count < max; count++)
-  {
-    event = [events objectAtIndex: count];
-    //      NSLog(@"***[UIxCalListingActions eventsBlocksAction] %i = %@ : %@ / %@ / %@", count,
-    //	    [event objectAtIndex: eventTitleIndex],
-    //	    [event objectAtIndex: eventStartDateIndex],
-    //	    [event objectAtIndex: eventEndDateIndex],
-    //	    [event objectAtIndex: eventRecurrenceIdIndex]);
-    eventNbr = [NSNumber numberWithUnsignedInt: count];
-    isAllDay = [[event objectAtIndex: eventIsAllDayIndex] boolValue];
-    if (dayBasedView && isAllDay)
-      [self _fillBlocks: allDayBlocks withEvent: event withNumber: eventNbr];
-    else
-      [self _fillBlocks: blocks withEvent: event withNumber: eventNbr];
-  }
+  events = [self _fetchFields: eventsFields forComponentOfType: @"vevent"];
   
-  max = [blocks count];
-  for (count = 0; count < max; count++)
+  if ([currentView isEqualToString: @"multicolumndayview"])
   {
-    currentDay = [blocks objectAtIndex: count];
-    [currentDay sortUsingSelector: @selector (compareEventByStart:)];
-    [self _addBlocksWidth: currentDay];
+    calendars = [self _selectedCalendars];
+    eventsByCalendars = [NSMutableArray arrayWithCapacity:[calendars count]];
+    for (i = 0; i < [calendars count]; i++) // For each calendar
+    {
+      eventsForCalendar = [NSMutableArray array];
+      [self _prepareEventBlocks: &blocks withAllDays: &allDayBlocks];
+      for (j = 0; j < [events count]; j++) {
+        if ([[[events objectAtIndex:j] objectAtIndex:1] isEqualToString:[calendars objectAtIndex:i]]) {
+          [eventsForCalendar addObject: [events objectAtIndex:j]];
+        }
+      }
+      eventsBlocks = [NSArray arrayWithObjects:eventsForCalendar, allDayBlocks, blocks, nil];
+      max = [eventsForCalendar count];
+      for (count = 0; count < max; count++)
+      {
+        event = [eventsForCalendar objectAtIndex: count];
+        eventNbr = [NSNumber numberWithUnsignedInt: count];
+        isAllDay = [[event objectAtIndex: eventIsAllDayIndex] boolValue];
+        if (dayBasedView && isAllDay)
+          [self _fillBlocks: allDayBlocks withEvent: event withNumber: eventNbr];
+        else
+          [self _fillBlocks: blocks withEvent: event withNumber: eventNbr];
+      }
+      max = [blocks count];
+      for (count = 0; count < max; count++)
+      {
+        currentDay = [blocks objectAtIndex: count];
+        [currentDay sortUsingSelector: @selector (compareEventByStart:)];
+        [self _addBlocksWidth: currentDay];
+      }
+      
+      [eventsByCalendars insertObject:eventsBlocks atIndex:i];
+    }
+    return [self _responseWithData: eventsByCalendars];
+  }
+  else
+  {
+    [self _prepareEventBlocks: &blocks withAllDays: &allDayBlocks];
+    eventsBlocks = [NSArray arrayWithObjects: events, allDayBlocks, blocks, nil];
+    max = [events count];
+    for (count = 0; count < max; count++)
+    {
+      event = [events objectAtIndex: count];
+      //      NSLog(@"***[UIxCalListingActions eventsBlocksAction] %i = %@ : %@ / %@ / %@", count,
+      //	    [event objectAtIndex: eventTitleIndex],
+      //	    [event objectAtIndex: eventStartDateIndex],
+      //	    [event objectAtIndex: eventEndDateIndex],
+      //	    [event objectAtIndex: eventRecurrenceIdIndex]);
+      eventNbr = [NSNumber numberWithUnsignedInt: count];
+      isAllDay = [[event objectAtIndex: eventIsAllDayIndex] boolValue];
+      if (dayBasedView && isAllDay)
+        [self _fillBlocks: allDayBlocks withEvent: event withNumber: eventNbr];
+      else
+        [self _fillBlocks: blocks withEvent: event withNumber: eventNbr];
+    }
+    
+    max = [blocks count];
+    for (count = 0; count < max; count++)
+    {
+      currentDay = [blocks objectAtIndex: count];
+      [currentDay sortUsingSelector: @selector (compareEventByStart:)];
+      [self _addBlocksWidth: currentDay];
+    }
+    return [self _responseWithData: eventsBlocks];
   }
-  
-  return [self _responseWithData: eventsBlocks];
-  //   timeIntervalSinceDate:
 }
 
 - (NSString *) _getStatusClassForStatusCode: (int) statusCode
@@ -1177,9 +1244,9 @@ _computeBlocksPosition (NSArray *blocks)
         [filteredTasks addObject: filteredTask];
       else if ([tasksView isEqualToString:@"view_all"])
         [filteredTasks addObject: filteredTask];
-      else if (([tasksView isEqualToString:@"view_overdue"]) && ([[filteredTask objectAtIndex:12] isEqualToString:@"overdue"]))
+      else if (([tasksView isEqualToString:@"view_overdue"]) && ([[filteredTask objectAtIndex:15] isEqualToString:@"overdue"]))
         [filteredTasks addObject: filteredTask];
-      else if ([tasksView isEqualToString:@"view_incomplete"] && (![[filteredTask objectAtIndex:12] isEqualToString:@"completed"]))
+      else if ([tasksView isEqualToString:@"view_incomplete"] && (![[filteredTask objectAtIndex:15] isEqualToString:@"completed"]))
         [filteredTasks addObject: filteredTask];
       else if ([tasksView isEqualToString:@"view_not_started"] && ([[[filteredTask objectAtIndex:3] stringValue] isEqualToString:@"0"]))
         [filteredTasks addObject: filteredTask];
@@ -1208,4 +1275,28 @@ _computeBlocksPosition (NSArray *blocks)
   return [self _responseWithData: filteredTasks];
 }
 
+- (WOResponse *) activeTasksAction
+{
+  NSMutableDictionary *activeTasksByCalendars;
+  SOGoAppointmentFolder *folder;
+  SOGoAppointmentFolders *co;
+  NSArray *folders;
+  
+  int i;
+  
+  co = [self clientObject];
+  folders = [co subFolders];
+  activeTasksByCalendars = [NSMutableDictionary dictionaryWithCapacity: [folders count]];
+
+  for (i = 0; i < [folders count]; i++)
+    {
+      folder = [folders objectAtIndex: i];
+      
+      [activeTasksByCalendars setObject: [folder activeTasks]
+                                 forKey: [folder nameInContainer]];
+  }
+  
+  return [self _responseWithData: activeTasksByCalendars];
+}
+
 @end
