@@ -1,9 +1,6 @@
 /* UIxComponentEditor.m - this file is part of SOGo
  *
- * Copyright (C) 2006-2013 Inverse inc.
- *
- * Author: Wolfgang Sourdeau <wsourdeau@inverse.ca>
- *         Francis Lachapelle <flachapelle@inverse.ca>
+ * Copyright (C) 2006-2015 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,10 +30,15 @@
 
 #import <NGCards/iCalAlarm.h>
 #import <NGCards/iCalByDayMask.h>
+#import <NGCards/iCalDateTime.h>
+#import <NGCards/iCalEvent.h>
 #import <NGCards/iCalPerson.h>
 #import <NGCards/iCalRepeatableEntityObject.h>
 #import <NGCards/iCalRecurrenceRule.h>
+#import <NGCards/iCalToDo.h>
 #import <NGCards/iCalTrigger.h>
+
+
 #import <NGCards/NSString+NGCards.h>
 #import <NGCards/NSCalendarDate+NGCards.h>
 #import <NGObjWeb/SoSecurityManager.h>
@@ -46,8 +48,10 @@
 #import <NGObjWeb/WOResponse.h>
 #import <NGExtensions/NSCalendarDate+misc.h>
 #import <NGExtensions/NSObject+Logs.h>
+#import <NGExtensions/NSNull+misc.h>
 #import <NGExtensions/NSString+misc.h>
 
+#import <Appointments/iCalAlarm+SOGo.h>
 #import <Appointments/iCalEntityObject+SOGo.h>
 #import <Appointments/iCalPerson+SOGo.h>
 #import <Appointments/SOGoAppointmentFolder.h>
@@ -172,7 +176,6 @@ iRANGE(2);
       [self setIsCycleEndNever];
       componentOwner = @"";
       organizer = nil;
-      //organizerIdentity = nil;
       organizerProfile = nil;
       ownerAsAttendee = nil;
       attendee = nil;
@@ -209,7 +212,6 @@ iRANGE(2);
   [title release];
   [location release];
   [organizer release];
-  //[organizerIdentity release];
   [organizerProfile release];
   [ownerAsAttendee release];
   [comment release];
@@ -222,7 +224,7 @@ iRANGE(2);
   [attendee release];
   [jsonAttendees release];
   [calendarList release];
-
+  
   [reminder release];
   [reminderQuantity release];
   [reminderUnit release];
@@ -531,16 +533,16 @@ iRANGE(2);
   if ([component hasAlarms])
     {
       // We currently have the following limitations for alarms:
-      // - only the first alarm is considered;
-      // - the alarm's action must be of type DISPLAY;
+      // - the alarm's action must be of type DISPLAY or AUDIO (considered as DISPLAY)
       // - the alarm's trigger value type must be DURATION.
 
-      anAlarm = [[component alarms] objectAtIndex: 0];
+      anAlarm = [component firstSupportedAlarm];
       aTrigger = [anAlarm trigger];
       ASSIGN (reminderAction, [[anAlarm action] lowercaseString]);
-      if (([reminderAction isEqualToString: @"display"]
-           || [reminderAction isEqualToString: @"email"])
-	  && [[aTrigger valueType] caseInsensitiveCompare: @"DURATION"] == NSOrderedSame)
+
+      //  The default value type is DURATION. See http://tools.ietf.org/html/rfc5545#section-3.8.6.3
+      if (![[aTrigger valueType] length] ||
+          [[aTrigger valueType] caseInsensitiveCompare: @"DURATION"] == NSOrderedSame)
 	{
 	  duration = [aTrigger flattenedValuesForKey: @""];
 	  i = [reminderValues indexOfObject: duration];
@@ -590,7 +592,7 @@ iRANGE(2);
 			      ASSIGN (reminderUnit, @"MINUTES");
 			      break;
 			    default:
-			      NSLog(@"Cannot process duration unit: '%c'", c);
+			      //NSLog(@"Cannot process duration unit: '%c'", c);
 			      break;
 			    }
 			}
@@ -680,12 +682,16 @@ iRANGE(2);
 	  ASSIGN (ownerAsAttendee, [component findAttendeeWithEmail: (id)ownerEmail]);
 	}
     }
-//   /* cycles */
-//   if ([component isRecurrent])
-//     {
-//       rrule = [[component recurrenceRules] objectAtIndex: 0];
-//       [self adjustCycleControlsForRRule: rrule];
-//     }
+}
+
+- (void) setRSVPURL: (NSString *) theURL
+{
+  rsvpURL = theURL;
+}
+
+- (NSString *) rsvpURL
+{
+  return rsvpURL;
 }
 
 - (void) setSaveURL: (NSString *) newSaveURL
@@ -812,25 +818,27 @@ iRANGE(2);
 	uid = email;
       
       [profile setObject: name
-			   forKey: @"name"];
+                  forKey: @"name"];
       
       [profile setObject: email
-			   forKey: @"email"];
+                  forKey: @"email"];
       
       if (partstat == nil || ![partstat length])
 	partstat = @"accepted";
+      
       [profile setObject: partstat
-			   forKey: @"partstat"];
+                  forKey: @"partstat"];
       
       if (role == nil || ![role length])
 	role = @"chair";
+      
       [profile setObject: role
-			   forKey: @"role"];
+                  forKey: @"role"];
       
       organizerProfile = [NSDictionary dictionaryWithObject: profile forKey: uid];
       [organizerProfile retain];
     }
-
+  
   return organizerProfile;
 }
 
@@ -889,30 +897,6 @@ iRANGE(2);
   return [[[[self organizerProfile] allValues] lastObject] jsonRepresentation];
 }
 
-// - (BOOL) canBeOrganizer
-// {
-//   NSString *owner;
-//   SOGoObject <SOGoComponentOccurence> *co;
-//   SOGoUser *currentUser;
-//   BOOL hasOrganizer;
-//   SoSecurityManager *sm;
-
-//   co = [self clientObject];
-//   owner = [co ownerInContext: context];
-//   currentUser = [context activeUser];
-
-//   hasOrganizer = ([[organizer value: 0] length] > 0);
-
-//   sm = [SoSecurityManager sharedSecurityManager];
-  
-//   return ([co isNew]
-// 	  || (([owner isEqualToString: [currentUser login]]
-// 	       || ![sm validatePermission: SOGoCalendarPerm_ModifyComponent
-// 		       onObject: co
-// 		       inContext: context])
-// 	      && (!hasOrganizer || [component userIsOrganizer: currentUser])));
-// }
-
 - (BOOL) hasOrganizer
 {
   // We check if there's an organizer and if it's not ourself
@@ -940,51 +924,8 @@ iRANGE(2);
     }
 
   return NO;
-
-    //return ([[organizer value: 0] length] && ![self canBeOrganizer]);
 }
 
-//- (void) setOrganizerIdentity: (NSDictionary *) newOrganizerIdentity
-//{
-//  ASSIGN (organizerIdentity, newOrganizerIdentity);
-//}
-
-// - (NSDictionary *) organizerIdentity
-// {
-//   NSArray *allIdentities;
-//   NSEnumerator *identities;
-//   NSDictionary *currentIdentity;
-//   NSString *orgEmail;
-
-//   orgEmail = [organizer rfc822Email];
-//   if (!organizerIdentity)
-//     {
-//       if ([orgEmail length])
-// 	{
-// 	  allIdentities = [[context activeUser] allIdentities];
-// 	  identities = [allIdentities objectEnumerator];
-// 	  while (!organizerIdentity
-// 		 && ((currentIdentity = [identities nextObject])))
-// 	    if ([[currentIdentity objectForKey: @"email"]
-// 		  caseInsensitiveCompare: orgEmail]
-// 		== NSOrderedSame)
-// 	      ASSIGN (organizerIdentity, currentIdentity);
-// 	}
-//     }
-
-//   return organizerIdentity;
-// }
-
-//- (NSArray *) organizerList
-//{
-//  return [[context activeUser] allIdentities];
-//}
-
-//- (NSString *) itemOrganizerText
-//{
-//  return [item keysWithFormat: @"%{fullName} <%{email}>"];
-//}
-
 - (BOOL) hasAttendees
 {
   return ([[component attendees] count] > 0);
@@ -1420,7 +1361,8 @@ iRANGE(2);
 
 - (void) setComponentCalendar: (SOGoAppointmentFolder *) _componentCalendar
 {
-  ASSIGN(componentCalendar, _componentCalendar);
+  if (_componentCalendar)
+    ASSIGN(componentCalendar, _componentCalendar);
 }
 
 /* priorities */
@@ -1460,16 +1402,16 @@ iRANGE(2);
 
 - (NSArray *) classificationClasses
 {
-  static NSArray *priorities = nil;
-
-  if (!priorities)
+  static NSArray *classes = nil;
+  
+  if (!classes)
     {
-      priorities = [NSArray arrayWithObjects: @"PUBLIC",
-                            @"CONFIDENTIAL", @"PRIVATE", nil];
-      [priorities retain];
+      classes = [NSArray arrayWithObjects: @"PUBLIC",
+                         @"CONFIDENTIAL", @"PRIVATE", nil];
+      [classes retain];
     }
-
-  return priorities;
+  
+  return classes;
 }
 
 - (void) setClassification: (NSString *) _classification
@@ -1712,19 +1654,14 @@ RANGE(2);
 - (BOOL) isWriteableClientObject
 {
   return [[self clientObject]
-	   respondsToSelector: @selector(saveContentString:)];
+	   respondsToSelector: @selector(saveCompontent:)];
 }
 
 /* access */
 
-- (BOOL) isMyComponent
-{
-  return ([[context activeUser] hasEmail: [organizer rfc822Email]]);
-}
-
 - (BOOL) canEditComponent
 {
-  return [self isMyComponent];
+  return ([[context activeUser] hasEmail: [organizer rfc822Email]]);
 }
 
 /* response generation */
@@ -1883,7 +1820,9 @@ RANGE(2);
 	  [component setAttendees: newAttendees];
 	}
       else
-	NSLog(@"Error scanning following JSON:\n%@", json);  
+        {
+	  //NSLog(@"Error scanning following JSON:\n%@", json);  
+        }
     }
 }
 
@@ -2170,40 +2109,7 @@ RANGE(2);
     }
 }
 
-- (void) _appendAttendees: (NSArray *) attendees
-             toEmailAlarm: (iCalAlarm *) alarm
-{
-  NSMutableArray *aAttendees;
-  int count, max;
-  iCalPerson *currentAttendee, *aAttendee;
 
-  max = [attendees count];
-  aAttendees = [NSMutableArray arrayWithCapacity: max];
-  for (count = 0; count < max; count++)
-    {
-      currentAttendee = [attendees objectAtIndex: count];
-      aAttendee = [iCalPerson elementWithTag: @"attendee"];
-      [aAttendee setCn: [currentAttendee cn]];
-      [aAttendee setEmail: [currentAttendee rfc822Email]];
-      [aAttendees addObject: aAttendee];
-    }
-  [alarm setAttendees: aAttendees];
-}
-
-- (void) _appendOrganizerToEmailAlarm: (iCalAlarm *) alarm
-{
-  NSString *uid;
-  NSDictionary *ownerIdentity;
-  iCalPerson *aAttendee;
-
-  uid = [[self clientObject] ownerInContext: context];
-  ownerIdentity = [[SOGoUser userWithLogin: uid roles: nil]
-                    defaultIdentity];
-  aAttendee = [iCalPerson elementWithTag: @"attendee"];
-  [aAttendee setCn: [ownerIdentity objectForKey: @"fullName"]];
-  [aAttendee setEmail: [ownerIdentity objectForKey: @"email"]];
-  [alarm addChild: aAttendee];
-}
 
 - (void) takeValuesFromRequest: (WORequest *) _rq
                      inContext: (WOContext *) _ctx
@@ -2238,68 +2144,43 @@ RANGE(2);
     [component removeAllAlarms];
   else
     {
-      iCalTrigger *aTrigger;
       iCalAlarm *anAlarm;
       NSString *aValue;
       NSUInteger index;
 
-      anAlarm = [iCalAlarm new];
-
       index = [reminderItems indexOfObject: reminder];
-
-      aTrigger = [iCalTrigger elementWithTag: @"TRIGGER"];
-      [aTrigger setValueType: @"DURATION"];
-      [anAlarm setTrigger: aTrigger];
-      
       aValue = [reminderValues objectAtIndex: index];
-      if ([aValue length]) {
-	// Predefined alarm
-        [anAlarm setAction: @"DISPLAY"];
-	[aTrigger setSingleValue: aValue forKey: @""];
-      }
-      else {
-	// Custom alarm
-        if ([reminderAction length] > 0 && [reminderUnit length] > 0)
-          {
-            [anAlarm setAction: [reminderAction uppercaseString]];
-            if ([reminderAction isEqualToString: @"email"])
-              {
-                [anAlarm removeAllAttendees];
-                if (reminderEmailAttendees)
-                  [self _appendAttendees: [component attendees]
-                        toEmailAlarm: anAlarm];
-                if (reminderEmailOrganizer)
-                  [self _appendOrganizerToEmailAlarm: anAlarm];
-                [anAlarm setSummary: [component summary]];
-                [anAlarm setComment: [component comment]];
-              }
-            
-            if ([reminderReference caseInsensitiveCompare: @"BEFORE"] == NSOrderedSame)
-              aValue = [NSString stringWithString: @"-P"];
-            else
-              aValue = [NSString stringWithString: @"P"];
-            
-            if ([reminderUnit caseInsensitiveCompare: @"MINUTES"] == NSOrderedSame ||
-                [reminderUnit caseInsensitiveCompare: @"HOURS"] == NSOrderedSame)
-              aValue = [aValue stringByAppendingString: @"T"];
-            
-            aValue = [aValue stringByAppendingFormat: @"%i%@",			 
-                             [reminderQuantity intValue],
-                             [reminderUnit substringToIndex: 1]];
-            [aTrigger setSingleValue: aValue forKey: @""];
-            [aTrigger setRelationType: reminderRelation];
-          }
-        else
-          {
-            [anAlarm release];
-            anAlarm = nil;
-          }
-      }
+
+      // Predefined alarm
+      if ([aValue length])
+        {
+          iCalTrigger *aTrigger;
+
+          anAlarm = [[[iCalAlarm alloc] init] autorelease];
+          aTrigger = [iCalTrigger elementWithTag: @"TRIGGER"];
+          [aTrigger setValueType: @"DURATION"];
+          [anAlarm setTrigger: aTrigger];
+          [anAlarm setAction: @"DISPLAY"];
+          [aTrigger setSingleValue: aValue forKey: @""];
+        }
+      else
+        {
+          // Custom alarm
+          anAlarm = [iCalAlarm alarmForEvent: component
+                                       owner: [[self clientObject] ownerInContext: context]
+                                      action: reminderAction
+                                        unit: reminderUnit
+                                    quantity: reminderQuantity
+                                   reference: reminderReference
+                            reminderRelation: reminderRelation
+                              emailAttendees: reminderEmailAttendees
+                              emailOrganizer: reminderEmailOrganizer];
+        }
+      
       if (anAlarm)
         {
           [component removeAllAlarms];
           [component addToAlarms: anAlarm];
-          [anAlarm release];
         }
     }
   
@@ -2599,4 +2480,9 @@ RANGE(2);
   return response;
 }
 
++ (NSArray *) reminderValues
+{
+  return reminderValues;
+}
+
 @end
