@@ -1,8 +1,6 @@
 /* UIxTaskEditor.m - this file is part of SOGo
  *
- * Copyright (C) 2007-2009 Inverse inc.
- *
- * Author: Wolfgang Sourdeau <wsourdeau@inverse.ca>
+ * Copyright (C) 2007-2014 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -43,6 +41,8 @@
 #import <SOGo/SOGoDateFormatter.h>
 #import <SOGo/SOGoUser.h>
 #import <SOGo/SOGoUserDefaults.h>
+
+#import <Appointments/iCalEntityObject+SOGo.h>
 #import <Appointments/SOGoAppointmentFolder.h>
 #import <Appointments/SOGoTaskObject.h>
 
@@ -91,7 +91,7 @@
 {
   if (!todo)
     {
-      todo = (iCalToDo *) [[self clientObject] component: YES secure: YES];
+      todo = (iCalToDo *) [[self clientObject] occurence];
       [[todo parent] retain];
     }
 
@@ -215,13 +215,12 @@
 
 - (BOOL) statusDateDisabled
 {
-  return ![status isEqualToString: @"COMPLETED"];
+  return !([status isEqualToString: @"COMPLETED"] || statusDate);
 }
 
 - (BOOL) statusPercentDisabled
 {
-  return ([status length] == 0
-	  || [status isEqualToString: @"CANCELLED"]);
+  return ([status length] == 0 || [status isEqualToString: @"CANCELLED"]);
 }
 
 - (void) setStatusPercent: (NSString *) newStatusPercent
@@ -313,11 +312,15 @@
         hasDueDate = YES;
       else
         dueDate = [self newStartDate];
+      
+      ASSIGN (statusDate, [todo completed]);
+      [statusDate setTimeZone: timeZone];
+
       ASSIGN (status, [todo status]);
-      if ([status isEqualToString: @"COMPLETED"])
+      
+      if ([status length] == 0 && statusDate)
 	{
-	  ASSIGN (statusDate, [todo completed]);
-	  [statusDate setTimeZone: timeZone];
+          ASSIGN(status, @"COMPLETED");
 	}
       else
 	{
@@ -421,29 +424,22 @@
   return [self jsCloseWithRefreshMethod: @"refreshTasks()"];
 }
 
-// - (id <WOActionResults>) saveAction
-// {
-//   SOGoTaskObject *clientObject;
-//   NSString *iCalString;
-
-//   clientObject = [self clientObject];
-//   iCalString = [[clientObject calendar: NO secure: NO] versitString];
-//   [clientObject saveContentString: iCalString];
-
-//   return [self jsCloseWithRefreshMethod: @"refreshTasks()"];
-// }
-
 - (id <WOActionResults>) viewAction
 {
   WOResponse *result;
   NSDictionary *data;
   NSCalendarDate *startDate, *dueDate;
+  SOGoCalendarComponent *co;
   NSTimeZone *timeZone;
   SOGoUserDefaults *ud;
+  iCalAlarm *anAlarm;
   BOOL resetAlarm;
+  BOOL snoozeAlarm;
 
   [self todo];
 
+  co = [self clientObject];
+
   result = [self responseWithStatus: 200];
   ud = [[context activeUser] userDefaults];
   timeZone = [ud timeZone];
@@ -451,21 +447,41 @@
   [startDate setTimeZone: timeZone];
   dueDate = [todo due];
   [dueDate setTimeZone: timeZone];
-  
-  resetAlarm = [[[context request] formValueForKey: @"resetAlarm"] boolValue];
-  if (resetAlarm && [todo hasAlarms] && ![todo hasRecurrenceRules])
-    {
-      iCalAlarm *anAlarm;
-      iCalTrigger *aTrigger;
-      SOGoCalendarComponent *co;
 
-      anAlarm = [[todo alarms] objectAtIndex: 0];
-      aTrigger = [anAlarm trigger];
-      [aTrigger setValue: 0 ofAttribute: @"x-webstatus" to: @"triggered"];
-      
-      co = [self clientObject];
-      [co saveComponent: todo];
+  // resetAlarm=yes is set only when we are about to show the alarm popup in the Web
+  // interface of SOGo. See generic.js for details. snoozeAlarm=X is called when the
+  // user clicks on "Snooze for" X minutes, when the popup is being displayed.
+  // If either is set to yes, we must find the right alarm.
+  resetAlarm = [[[context request] formValueForKey: @"resetAlarm"] boolValue];
+  snoozeAlarm = [[[context request] formValueForKey: @"snoozeAlarm"] intValue];
+  
+  if (resetAlarm || snoozeAlarm)
+    {
+      iCalToDo *master;
+
+      master = todo;
+      [[co container] findEntityForClosestAlarm: &todo
+                                       timezone: timeZone
+                                      startDate: &startDate
+                                        endDate: &dueDate];
+
+      anAlarm = [todo firstDisplayOrAudioAlarm];
+
+      if (resetAlarm)
+        {
+          iCalTrigger *aTrigger;
+          
+          aTrigger = [anAlarm trigger];
+          [aTrigger setValue: 0 ofAttribute: @"x-webstatus" to: @"triggered"];
+          
+          [co saveComponent: master];
+        }
+      else if (snoozeAlarm)
+        {
+          [co snoozeAlarm: snoozeAlarm];
+        }
     }
+  resetAlarm = [[[context request] formValueForKey: @"resetAlarm"] boolValue];
   
   data = [NSDictionary dictionaryWithObjectsAndKeys:
 		       [todo tag], @"component",
@@ -552,26 +568,6 @@
 
 }
 
-// TODO: add tentatively
-
-// - (id) acceptOrDeclineAction: (BOOL) _accept
-// {
-//   [[self clientObject] changeParticipationStatus:
-//                          _accept ? @"ACCEPTED" : @"DECLINED"];
-
-//   return self;
-// }
-
-// - (id) acceptAction
-// {
-//   return [self acceptOrDeclineAction: YES];
-// }
-
-// - (id) declineAction
-// {
-//   return [self acceptOrDeclineAction: NO];
-// }
-
 - (id) changeStatusAction
 {
   NSString *newStatus;
