@@ -11,6 +11,8 @@ var Contact = {
     currentContactId: null
 };
 
+var refreshViewCheckTimer;
+
 function openContactsFolder(contactsFolder, reload, idx) {
     if ((contactsFolder && contactsFolder != Contact.currentAddressBook)
         || reload) {
@@ -53,7 +55,7 @@ function contactsListCallback(http) {
     if (http.readyState == 4) {
         if (http.status == 200) {
             document.contactsListAjaxRequest = null;
-            
+
             var div = $("contactsListContent");
             var table = $("contactsList");
             var tbody = table.tBodies[0];
@@ -173,13 +175,13 @@ function contactsListCallback(http) {
                     sortHeader = $("phoneHeader");
                 else
                     sortHeader = null;
-                
+
                 if (sortHeader) {
                     var sortImages = $(table.tHead).select(".sortImage");
                     $(sortImages).each(function(item) {
                             item.remove();
                         });
-                    
+
                     var sortImage = createElement("img", "messageSortImage", "sortImage");
                     sortHeader.insertBefore(sortImage, sortHeader.firstChild);
                     if (sorting["ascending"])
@@ -433,7 +435,7 @@ function onContactSelectionChange(event) {
     }
     if (contactView) {
         var rows = this.parentNode.getSelectedRowsId();
-  
+
         if (rows.length == 1) {
             var node = $(rows[0]);
             loadContact(node.getAttribute('id'));
@@ -499,7 +501,7 @@ function onToolbarWriteToSelectedContacts(event) {
         if (document.body.hasClassName("popup"))
             window.close();
     }
-  
+
     return false;
 }
 
@@ -630,6 +632,9 @@ function onFolderSelectionChange(event) {
         var node = getTarget(event);
         if (node.tagName == 'UL')
             return;
+        if (node.tagName == "SPAN")
+            node = node.parentNode;
+
         // Update rows selection
         onRowClick(event, node);
     }
@@ -661,7 +666,7 @@ function onConfirmContactSelection(event) {
     var currentAddressBookName = folder.textContent;
     var selectorList = null;
     var initialValues = null;
-	
+
     var contactsList = $("contactsList");
     var rows = contactsList.getSelectedRows();
     for (i = 0; i < rows.length; i++) {
@@ -754,9 +759,13 @@ function appendAddressBook(name, folder) {
         li.setAttribute("list-editing", "available");
         li.setAttribute("acl-editing", "available");
         li.addClassName("local");
-        li.appendChild(document.createTextNode(name
-                                               .replace("&lt;", "<", "g")
-                                               .replace("&gt;", ">", "g")));
+
+        var displayName = document.createElement("span");
+        displayName.appendChild(document.createTextNode(name
+                                                        .replace("&lt;", "<", "g")
+                                                        .replace("&gt;", ">", "g")));
+        li.appendChild(displayName);
+
         updateAddressBooksMenus();
         configureDroppables();
     }
@@ -780,7 +789,7 @@ function onFolderUnsubscribeCB(folderId) {
     var node = $(folderId);
     node.deselect();
     node.parentNode.removeChild(node);
-    
+
     var personal = $("/personal");
     personal.selectElement();
     onFolderSelectionChange();
@@ -917,7 +926,7 @@ function deletePersonalAddressBookCallback(http) {
     if (http.readyState == 4) {
         if (isHttpStatus204(http.status)) {
             var ul = $("contactFolders");
-	
+
             var children = ul.childNodesWithTag("li");
             var i = 0;
             var done = false;
@@ -995,7 +1004,7 @@ function configureAddressBooks() {
         contactFolders.on("dblclick", onAddressBookModify);
         contactFolders.on("selectstart", listRowMouseDownHandler);
         contactFolders.attachMenu("contactFoldersMenu");
-    
+
         lookupDeniedFolders();
         configureDroppables();
 
@@ -1034,16 +1043,16 @@ function updateAddressBooksMenus() {
             var menuDIV = $(menuId);
             if (menuDIV)
                 menuDIV.parentNode.removeChild(menuDIV);
-	
+
             menuDIV = document.createElement("div");
             pageContent.appendChild(menuDIV);
-	
+
             var menu = document.createElement("ul");
             menuDIV.appendChild(menu);
-	
+
             $(menuDIV).addClassName("menu");
             menuDIV.setAttribute("id", menuId);
-	
+
             var submenuIds = new Array();
             for (var i = 0; i < contactFolders.length; i++) {
                 if (contactFolders[i].hasClassName("local")) {
@@ -1060,21 +1069,22 @@ function updateAddressBooksMenus() {
         }
     }
 }
-  
+
 function onAddressBookModify(event) {
     var folders = $("contactFolders");
     var selected = folders.getSelectedNodes()[0];
-    var addressBookID = selected.getAttribute("id");
-    var url = ApplicationBaseURL + addressBookID + "/properties";
-    var windowID = sanitizeWindowName(addressBookID + " properties");
-    var width = 410;
-    var height = 410;
-  
-    $(function() {
-            var properties = window.open(url, windowID, "width="+width+",height="+height+",resizable=0");
-            properties.focus();
-        }).delay(0.1);
-  
+    if (selected.getAttribute("list-editing") == "available") {
+        var addressBookID = selected.getAttribute("id");
+        var url = ApplicationBaseURL + addressBookID + "/properties";
+        var windowID = sanitizeWindowName(addressBookID + " properties");
+        var width = 410;
+        var height = 410;
+        
+        $(function() {
+          var properties = window.open(url, windowID, "width="+width+",height="+height+",resizable=0");
+          properties.focus();
+          }).delay(0.1);
+    }
 }
 
 function onMenuSharing(event) {
@@ -1103,19 +1113,14 @@ function onAddressBooksMenuPrepareVisibility() {
 
         var menu = $("contactFoldersMenu").down("ul");;
         var listElements = menu.childNodesWithTag("li");
-        var modifyOption = listElements[0];
         var newListOption = listElements[3];
         var removeOption = listElements[5];
         var exportOption = listElements[7];
         var importOption = listElements[8];
         var sharingOption = listElements[listElements.length - 1];
 
-        // Disable the "Sharing" and "Modify" options when address book
-        // is not owned by user
+        // Disable the "Sharing" option when address book is not owned by user
         if (folderOwner == UserLogin || IsSuperUser) {
-            modifyOption.removeClassName("disabled"); // WARNING: will fail
-                                                      // for super users
-                                                      // anyway
             var aclEditing = selected[0].getAttribute("acl-editing");
             if (aclEditing && aclEditing == "available") {
                 sharingOption.removeClassName("disabled");
@@ -1137,8 +1142,7 @@ function onAddressBooksMenuPrepareVisibility() {
             newListOption.addClassName("disabled");
         }
 
-        /* Disable the "remove" and "export ab" options when address book is
-           public */
+        // Disable the "remove" and "export ab" options when address book is public
         if (folderOwner == "nobody") {
             exportOption.addClassName("disabled");
             importOption.addClassName("disabled");
@@ -1246,14 +1250,14 @@ getMenus = function() {
                                      "moveContactMenu", "copyContactMenu", 
                                      onMenuExportContact, onMenuRawContact);
     menus["searchMenu"] = new Array(setSearchCriteria, setSearchCriteria, setSearchCriteria);
-   
+
     var contactFoldersMenu = $("contactFoldersMenu");
     if (contactFoldersMenu)
         contactFoldersMenu.prepareVisibility = onAddressBooksMenuPrepareVisibility;
     var contactMenu = $("contactMenu");
     if (contactMenu)
         contactMenu.prepareVisibility = onContactMenuPrepareVisibility;
-   
+
     if (originalGetMenus) {
         var originalMenus = originalGetMenus();
         if (originalMenus)
@@ -1299,7 +1303,7 @@ function onDocumentKeydown(event) {
                     nextRow = row.previous("tr");
                 if (nextRow) {
                     row.up().deselectAll();
-                    
+
                     // Adjust the scollbar
                     var viewPort = $("contactsListContent");
                     var divDimensions = viewPort.getDimensions();
@@ -1312,7 +1316,7 @@ function onDocumentKeydown(event) {
                         viewPort.scrollTop += rowBottom - divBottom;
                     else if (rowScrollOffset.top > rowPosition.top)
                         viewPort.scrollTop -= rowScrollOffset.top - rowPosition.top;
-                    
+
                     // Select and load the next message
                     nextRow.selectElement();
                     loadContact(nextRow.readAttribute("id"));
@@ -1351,10 +1355,11 @@ function initContacts(event) {
     }
 
     Event.observe(document, "keydown", onDocumentKeydown);
-    
+
     configureAddressBooks();
     configureDraggables();
     updateAddressBooksMenus();
+    initRefreshViewCheckTimer();
 
     var table = $("contactsList");
     if (table) {
@@ -1371,7 +1376,7 @@ function initContacts(event) {
         }
         configureSortableTableHeaders(table);
     }
-    
+
     if (typeof onWindowResize != 'function') {
         // When loaded from the mail editor, onWindowResize is
         // already registered
@@ -1617,4 +1622,24 @@ function onContactsReload () {
     openContactsFolder(Contact.currentAddressBook, true);
 }
 
+function initRefreshViewCheckTimer() {
+  var refreshViewCheck = typeof UserDefaults == 'undefined' ? false : UserDefaults["SOGoRefreshViewCheck"];
+  if (refreshViewCheck && refreshViewCheck != "manually") {
+    var interval;
+    if (refreshViewCheck == "once_per_hour")
+      interval = 3600;
+    else if (refreshViewCheck == "every_minute")
+      interval = 60;
+    else {
+      interval = parseInt(refreshViewCheck.substr(6)) * 60;
+    }
+    refreshViewCheckTimer = window.setInterval(onRefreshViewCheckCallback,
+                                               interval * 1000);
+  }
+}
+
+function onRefreshViewCheckCallback(event) {
+  onContactsReload();
+}
+
 document.observe("dom:loaded", initContacts);
