@@ -3,111 +3,145 @@ var mailAccounts = null;
 var dialogs = {};
 
 function savePreferences(sender) {
-  var sendForm = true;
-  
-  var sigList = $("signaturePlacementList");
-  if (sigList)
-    sigList.disabled = false;
-  
-  if ($("calendarCategoriesListWrapper"))
-    serializeCalendarCategories();
-  
-  if ($("contactsCategoriesListWrapper"))
-    serializeContactsCategories();
-  
-  if ($("mailLabelsListWrapper"))
-    serializeMailLabels();
-  
-  if (typeof mailCustomFromEnabled !== "undefined" && !emailRE.test($("email").value)) {
-    showAlertDialog(_("Please specify a valid sender address."));
-    sendForm = false;
-  }
-  
-  if ($("replyTo")) {
-    var replyTo = $("replyTo").value;
-    if (!replyTo.blank() && !emailRE.test(replyTo)) {
-      showAlertDialog(_("Please specify a valid reply-to address."));
-      sendForm = false;
-    }
-  }
-  
-  if ($("dayStartTime")) {
-    var start = $("dayStartTime");
-    var selectedStart = parseInt(start.options[start.selectedIndex].value);
-    var end = $("dayEndTime");
-    var selectedEnd = parseInt(end.options[end.selectedIndex].value);
-    
-    if (selectedStart >= selectedEnd) {
-      showAlertDialog (_("Day start time must be prior to day end time."));
-      sendForm = false;
-    }
-  }
-  
-  if ($("enableVacation") && $("enableVacation").checked) {
-    if ($("autoReplyText").value.strip().length == 0 || $("autoReplyEmailAddresses").value.strip().length == 0) {
-      showAlertDialog(_("Please specify your message and your email addresses for which you want to enable auto reply."));
-      sendForm = false;
-    }
-    
-    if ($("autoReplyText").value.strip().endsWith('\n.')) {
-      showAlertDialog(_("Your vacation message must not end with a single dot on a line."));
-	    sendForm = false;
-    }
-    
-    if ($("enableVacationEndDate") && $("enableVacationEndDate").checked) {
-      var e = $("vacationEndDate_date");
-      var endDate = e.inputAsDate();
-      var now = new Date();
-      if (isNaN(endDate.getTime()) || endDate.getTime() < now.getTime()) {
-        showAlertDialog(_("End date of your auto reply must be in the future."));
+    var sendForm = true;
+
+    var sigList = $("signaturePlacementList");
+    if (sigList)
+        sigList.disabled = false;
+
+    if ($("appointmentsWhiteListWrapper"))
+        serializeAppointmentsWhiteList();
+
+    if ($("calendarCategoriesListWrapper"))
+        serializeCalendarCategories();
+
+    if ($("contactsCategoriesListWrapper"))
+        serializeContactsCategories();
+
+    if ($("mailLabelsListWrapper"))
+        serializeMailLabels();
+
+    if (typeof mailCustomFromEnabled !== "undefined" && !emailRE.test($("email").value)) {
+        showAlertDialog(_("Please specify a valid sender address."));
         sendForm = false;
-      }
     }
-  }
-  
-  if ($("enableForward") && $("enableForward").checked) {
-    var addresses = $("forwardAddress").value.split(",");
-    for (var i = 0; i < addresses.length && sendForm; i++)
-      if (!emailRE.test(addresses[i].strip())) {
-        showAlertDialog(_("Please specify an address to which you want to forward your messages."));
-        sendForm = false;
-      }
-  }
-  
-  if (typeof sieveCapabilities != "undefined") {
-    var jsonFilters = prototypeIfyFilters();
-    $("sieveFilters").setValue(Object.toJSON(jsonFilters));
-  }
-  
-  if (sendForm) {
-    saveMailAccounts();
-    
-    triggerAjaxRequest($("mainForm").readAttribute("action"), function (http) {
-      if (http.readyState == 4) {
-        var response = http.responseText.evalJSON(true);
-        if (http.status == 503) {
-          showAlertDialog(_(response.textStatus));
+
+    if ($("replyTo")) {
+        var replyTo = $("replyTo").value;
+        if (!replyTo.blank() && !emailRE.test(replyTo)) {
+            showAlertDialog(_("Please specify a valid reply-to address."));
+            sendForm = false;
         }
-        else if (http.status == 200) {
-          if (response.hasChanged == 1) {
-            window.opener.location.reload();
-            window.close();
-          }
-          else {
-            window.close();
-          }
+    }
+
+    if ($("dayStartTime")) {
+        var start = $("dayStartTime");
+        var selectedStart = parseInt(start.options[start.selectedIndex].value);
+        var end = $("dayEndTime");
+        var selectedEnd = parseInt(end.options[end.selectedIndex].value);
+
+        if (selectedStart >= selectedEnd) {
+            showAlertDialog (_("Day start time must be prior to day end time."));
+            sendForm = false;
         }
-        else {
-          showAlertDialog(_(response.textStatus));
+    }
+
+    if ($("enableVacation") && $("enableVacation").checked) {
+        if ($("autoReplyText").value.strip().length == 0 || $("autoReplyEmailAddresses").value.strip().length == 0) {
+            showAlertDialog(_("Please specify your message and your email addresses for which you want to enable auto reply."));
+            sendForm = false;
         }
-      }
-    },
-    null,
-    Form.serialize($("mainForm")), // excludes the file input
-    { "Content-type": "application/x-www-form-urlencoded"}
-    );
-	}
-  return false;
+
+        if ($("autoReplyText").value.strip().endsWith('\n.')) {
+            showAlertDialog(_("Your vacation message must not end with a single dot on a line."));
+            sendForm = false;
+        }
+
+        if ($("enableVacationEndDate") && $("enableVacationEndDate").checked) {
+            var e = $("vacationEndDate_date");
+            var endDate = e.inputAsDate();
+            var now = new Date();
+            if (isNaN(endDate.getTime()) || endDate.getTime() < now.getTime()) {
+                showAlertDialog(_("End date of your auto reply must be in the future."));
+                sendForm = false;
+            }
+        }
+    }
+
+    if ($("enableForward") && $("enableForward").checked) {
+        var addresses = $("forwardAddress").value.split(",");
+
+        // We check if all addresses are valid
+        for (var i = 0; i < addresses.length && sendForm; i++)
+            if (!emailRE.test(addresses[i].strip())) {
+                showAlertDialog(_("Please specify an address to which you want to forward your messages."));
+                sendForm = false;
+            }
+       
+        // We check if we can only to internal/external addresses.
+        var constraints = parseInt(forwardConstraints);
+        
+        if (constraints > 0) {
+            // We first extract the list of 'known domains' to SOGo
+            var defaultAddresses = $("defaultEmailAddresses").value.split(/, */);
+            var domains = new Array();
+            
+            defaultAddresses.each(function(adr) {
+                var domain = adr.split("@")[1];
+                if (domain) {
+                    domains.push(domain.toLowerCase());
+                }
+            });
+
+            // We check if we're allowed or not to forward based on the domain defaults
+            for (var i = 0; i < addresses.length && sendForm; i++) {
+                var domain = addresses[i].split("@")[1].toLowerCase();
+                if (domains.indexOf(domain) < 0 && constraints == 1) {
+                    showAlertDialog(_("You are not allowed to forward your messages to an external email address."));
+                    sendForm = false;
+                }
+                else if (domains.indexOf(domain) >= 0 && constraints == 2) {
+                    showAlertDialog(_("You are not allowed to forward your messages to an internal email address."));
+                    sendForm = false;
+                }
+            }
+        }
+    }
+
+    if (typeof sieveCapabilities != "undefined") {
+        var jsonFilters = prototypeIfyFilters();
+        $("sieveFilters").setValue(Object.toJSON(jsonFilters));
+    }
+
+    if (sendForm) {
+        saveMailAccounts();
+
+        triggerAjaxRequest($("mainForm").readAttribute("action"), function (http) {
+            if (http.readyState == 4) {
+                var response = http.responseText.evalJSON(true);
+                if (http.status == 503) {
+                    showAlertDialog(_(response.textStatus));
+                }
+                else if (http.status == 200) {
+                    if (response.hasChanged == 1) {
+                        window.opener.location.reload();
+                        window.close();
+                    }
+                    else {
+                        window.close();
+                    }
+                }
+                else {
+                    showAlertDialog(_(response.textStatus));
+                }
+            }
+        },
+                           null,
+                           Form.serialize($("mainForm")), // excludes the file input
+                           { "Content-type": "application/x-www-form-urlencoded"}
+                          );
+    }
+    return false;
 }
 
 function prototypeIfyFilters() {
@@ -141,7 +175,7 @@ function prototypeIfyFilters() {
 function _setupEvents() {
     var widgets = [ "timezone", "shortDateFormat", "longDateFormat",
                     "timeFormat", "weekStartDay", "dayStartTime", "dayEndTime",
-                    "firstWeek", "messageCheck", "sortByThreads", "displayRemoteInlineImages",
+                    "firstWeek", "refreshViewCheck", "sortByThreads", "displayRemoteInlineImages",
                     "subscribedFoldersOnly", "language", "defaultCalendar",
                     "enableVacation" ];
     for (var i = 0; i < widgets.length; i++) {
@@ -212,6 +246,13 @@ function initPreferences() {
         mailController.attachToTabsContainer(tabsContainer);
     }
 
+    // Inner tabs on the calendar module tab
+    tabsContainer = $('calendarOptionsTabs');
+    if (tabsContainer) {
+        var mailController = new SOGoTabsController();
+        mailController.attachToTabsContainer(tabsContainer);
+    }
+
     _setupEvents();
 
     // Optional function called when initializing the preferences
@@ -223,6 +264,57 @@ function initPreferences() {
     $('colorPickerDialog').on('click', 'span', onColorPickerChoice);
     $(document.body).on("click", onBodyClickHandler);
 
+    // Calendar whiteList
+    var whiteList = $("appointmentsWhiteListWrapper");
+    if (whiteList) {
+        var whiteListString = $("whiteList").getValue();
+        var whiteListObject = {};
+        // This condition is a backward compatibility where the strings looks like : "sogo1=John DOE <sogo1@example.com>"
+        if (whiteListString.search("=") != -1) {
+            var split = whiteListString.split("=");
+            whiteListObject[split[0]] = split[1];
+            
+        }
+        else if (whiteListString != "") {
+            whiteListObject = JSON.parse(whiteListString);
+        }
+        var allKeys = Object.keys(whiteListObject);
+        var allValues = Object.values(whiteListObject);
+        var tablebody = $("appointmentsWhiteListWrapper").childNodesWithTag("table")[0].tBodies[0];
+        for (i = 0; i < allKeys.length; i++) {
+            var row = new Element("tr");
+            var td = new Element("td").update("");
+            var textField = new Element("input");
+            var span = new Element("span");
+            
+            row.addClassName("whiteListRow");
+            row.observe("mousedown", onRowClick);
+            td.addClassName ("whiteListCell");
+            td.observe("mousedown", endAllEditables);
+            td.observe("dblclick", onNameEdit);
+            textField.addInterface(SOGoAutoCompletionInterface);
+            textField.SOGoUsersSearch = true;
+            textField.observe("autocompletion:changed", endEditable);
+            textField.addClassName("textField");
+            textField.value = allValues[i];
+            textField.setAttribute("uid", allKeys[i]);
+            textField.hide();
+            span.innerText = allValues[i];
+            
+            td.appendChild(textField);
+            td.appendChild(span);
+            row.appendChild (td);
+            tablebody.appendChild(row);
+            $(tablebody).deselectAll();
+            
+        }
+        
+        var table = whiteList.childNodesWithTag("table")[0];
+        table.multiselect = true;
+        $("appointmentsWhiteListAdd").observe("click", onAppointmentsWhiteListAdd);
+        $("appointmentsWhiteListDelete").observe("click", onAppointmentsWhiteListDelete);
+    }
+
     // Calender categories
     var wrapper = $("calendarCategoriesListWrapper");
     if (wrapper) {
@@ -282,8 +374,7 @@ function initPreferences() {
 
     button = $("enableVacationEndDate");
     if (button) {
-        jQuery("#vacationEndDate_date").closest(".date").datepicker(
-            { autoclose: true, position: 'above', weekStart: $('weekStartDay').getValue() });
+        jQuery("#vacationEndDate_date").closest(".date").datepicker({ autoclose: true, position: 'above', weekStart: $('weekStartDay').getValue() });
         button.on("click", function(event) {
             if (this.checked)
                 $("vacationEndDate_date").enable();
@@ -291,6 +382,7 @@ function initPreferences() {
                 $("vacationEndDate_date").disable();
         });
     }
+    onAddOutgoingAddressesCheck();
 }
 
 function initSieveFilters() {
@@ -618,7 +710,7 @@ function onMailIdentitySignatureClick(event) {
                                  { height: "150px",
                                    toolbar: [['Bold', 'Italic', '-', 'Link',
                                               'Font','FontSize','-','TextColor',
-                                              'BGColor']
+                                              'BGColor'], ['Source']
                                             ],
                                    language: localeCode,
                                    scayt_sLang: localeCode });
@@ -883,10 +975,10 @@ function saveMailAccounts() {
 
     // Could be null if ModuleConstraints disables email access
     if (editor)
-    	//Instead of removing the modules, we disable it. This will prevent the window to crash if we have a connection error.
-      editor.select('input, select').each(function(i) { i.disable(); })
+        //Instead of removing the modules, we disable it. This will prevent the window to crash if we have a connection error.
+        editor.select('input, select').each(function(i) { i.disable(); })
 
-    compactMailAccounts();
+            compactMailAccounts();
     var mailAccountsJSON = $("mailAccountsJSON");
 
     if (mailAccountsJSON)
@@ -976,6 +1068,118 @@ function onCalendarColorEdit(e) {
     onCCE(e, "calendarCategoriesListWrapper");
 }
 
+function makeEditable (element) {
+    element.addClassName("editing");
+    element.removeClassName("whiteListCell");
+
+    var span = element.down("SPAN");
+    span.update();
+
+    var textField = element.down("INPUT");
+    textField.show();
+    textField.focus();
+    textField.select();
+
+    return true;
+}
+
+function endAllEditables (e) {
+    var r = $$("TABLE#tableViewWhiteList TBODY TR TD");
+    for (var i = 0; i < r.length; i++) {
+        var element = $(r[i]);
+        if (r[i] != this && element.hasClassName("editing"))
+            endEditable(null, element.down("INPUT"));
+    }
+}
+
+function onNameEdit (e) {
+    endAllEditables();
+    if (!this.hasClassName("editing")) {
+        makeEditable (this);
+    }
+}
+
+function endEditable(event, textField) {
+    if (!textField)
+        textField = this;
+
+    var uid = textField.readAttribute("uid");
+    var cell = textField.up("TD");
+    var textSpan = cell.down("SPAN");
+
+    cell.removeClassName("editing");
+    cell.addClassName("whiteListCell");
+    textField.hide();
+
+    var tmp = textField.value;
+    tmp = tmp.replace (/</, "&lt;");
+    tmp = tmp.replace (/>/, "&gt;");
+    if (!uid)
+        cell.up("TR").addClassName("notfound");
+    if (tmp)
+        textSpan.update(tmp);
+    else
+        cell.up("TR").remove();
+
+    if (event)
+        Event.stop(event);
+
+    return false;
+}
+
+function onAppointmentsWhiteListAdd(e) {
+    var tablebody = $("appointmentsWhiteListWrapper").childNodesWithTag("table")[0].tBodies[0];
+    var row = new Element("tr");
+    var td = new Element("td").update("");
+    var textField = new Element("input");
+    var span = new Element("span");
+
+    row.addClassName("whiteListRow");
+    row.observe("mousedown", onRowClick);
+    td.addClassName ("whiteListCell");
+    td.observe("mousedown", endAllEditables);
+    td.observe("dblclick", onNameEdit);
+    textField.addInterface(SOGoAutoCompletionInterface);
+    textField.SOGoUsersSearch = true;
+    textField.observe("autocompletion:changed", endEditable);
+    textField.addClassName("textField");
+
+    td.appendChild(textField);
+    td.appendChild(span);
+    row.appendChild (td);
+    tablebody.appendChild(row);
+    $(tablebody).deselectAll();
+    row.selectElement();
+
+    makeEditable(td);
+
+}
+
+function onAppointmentsWhiteListDelete(e) {
+    var list = $('appointmentsWhiteListWrapper').down("TABLE").down("TBODY");
+    var rows = list.getSelectedNodes();
+    var count = rows.length;
+
+    for (var i=0; i < count; i++) {
+        rows[i].editionController = null;
+        rows[i].remove();
+    }
+}
+
+function serializeAppointmentsWhiteList() {
+    var r = $$("#appointmentsWhiteListWrapper TBODY TR");
+
+    var users = {};
+    for (var i = 0; i < r.length; i++) {
+        var tds = r[i].childElements().first().down("INPUT");
+        var uid  = tds.getAttribute("uid");
+        var value = tds.getValue();
+        if (uid != null)
+            users[uid] = value;
+    }
+    $("whiteList").value = Object.toJSON(users);
+}
+
 function onCalendarCategoryAdd(e) {
     var row = new Element("tr");
     var nametd = new Element("td").update("");
@@ -1015,7 +1219,7 @@ function serializeCalendarCategories() {
     var values = [];
     for (var i = 0; i < r.length; i++) {
         var tds = r[i].childElements();
-        var name  = $(tds.first()).innerHTML;
+        var name  = $(tds.first()).innerHTML.trim();
         var color = $(tds.last().childElements().first()).readAttribute('data-color');
         values.push("\"" + name + "\": \"" + color + "\"");
     }
@@ -1105,7 +1309,7 @@ function serializeMailLabels() {
         var name = r[i].readAttribute("data-name"); 
         var label  = $(tds.first()).innerHTML;
         var color = $(tds.last().childElements().first()).readAttribute('data-color');
-        
+
         /* if name is null, that's because we've just added a new tag */
         if (!name) {
             name = label.replace(/[ \(\)\/\{%\*<>\\\"]/g, "_");
@@ -1143,7 +1347,7 @@ function onContactsCategoryAdd(e) {
     var list = $('contactsCategoriesListWrapper').down("TABLE").down("TBODY");
     list.appendChild(row);
 
-    resetContactsTableActions ();
+    resetContactsTableActions();
     nametd.editionController.startEditing();
 }
 
@@ -1173,6 +1377,12 @@ function serializeContactsCategories() {
 
 /* / contact categories */
 
+function onAddOutgoingAddressesCheck(checkBox) {
+    if (!checkBox) {
+        checkBox = $("addOutgoingAddresses");
+    }
+    $("addressBookList").disabled = !checkBox.checked;
+}
 
 function onReplyPlacementListChange() {
     if ($("replyPlacementList").value == 0) {
